package gov.va.vss.config;

import javax.jms.ConnectionFactory;
import javax.naming.NamingException;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.env.Environment;
import org.springframework.jms.annotation.EnableJms;
import org.springframework.jms.config.DefaultJmsListenerContainerFactory;
import org.springframework.jms.core.JmsTemplate;

@Configuration
@EnableJms
public class MessagingConfig {

	@Autowired
	private Environment env;
	@Autowired
	private ConnectionFactory jmsConnectionFactory;

	@Bean
	public DefaultJmsListenerContainerFactory jmsListenerContainerFactory()
			throws IllegalArgumentException, NamingException {
		DefaultJmsListenerContainerFactory factory = new DefaultJmsListenerContainerFactory();
		factory.setConnectionFactory(jmsConnectionFactory);
		factory.setConcurrency("3-10");
		factory.setCacheLevelName("CACHE_NONE");
		return factory;
	}

	// @Bean
	// public JndiTemplate jndiTemplate() {
	// JndiTemplate jndiTemplate = new JndiTemplate();
	// Properties jndiProps = new Properties();
	//
	// for (String prop : new String[] { "java.naming.factory.initial",
	// "java.naming.provider.url",
	// // "java.naming.security.principal", // username
	// // "java.naming.security.credentials" // password
	// })
	// jndiProps.setProperty(prop, env.getProperty(prop));
	// jndiTemplate.setEnvironment(jndiProps);
	// return jndiTemplate;
	// }

	@Bean
	public JmsTemplate jmsTemplateBatchProcessEvents() throws IllegalArgumentException, NamingException {
		JmsTemplate t = new JmsTemplate();
		t.setConnectionFactory(jmsConnectionFactory);
		t.setDefaultDestinationName(env.getProperty("jms.batchProcess.queueName"));
		return t;
	}

	@Bean
	public JmsTemplate jmsTemplateCommonEvents() throws IllegalArgumentException, NamingException {
		JmsTemplate t = new JmsTemplate();
		t.setConnectionFactory(jmsConnectionFactory);
		t.setDefaultDestinationName(env.getProperty("jms.commonEvents.queueName"));
		return t;
	}

}
