package gov.va.vss.model;

import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.NONE;
import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.PUBLIC_ONLY;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;

import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;
import org.hibernate.validator.constraints.Length;
import org.hibernate.validator.constraints.NotBlank;

import com.fasterxml.jackson.annotation.JsonAutoDetect;

import gov.va.shared.model.core.AbstractAuditedVersionedPersistent;
import gov.va.vss.model.lookup.WorkdaySchedule;

@Entity
@Table(name = "VtkVolunteerCombinationAssignments", schema = "dbo")
@JsonAutoDetect(fieldVisibility = NONE, getterVisibility = PUBLIC_ONLY, isGetterVisibility = PUBLIC_ONLY)
public class CombinationCode extends AbstractAuditedVersionedPersistent<CombinationCode> {
	private static final long serialVersionUID = -8641562840185748448L;

	// -------------------------------------- Fields

	@NotNull
	private VolunteerStationProfile stationProfile;
	@NotNull
	private Organization organization;
	@NotNull
	private WorkdaySchedule schedule;
	@NotNull
	private BenefitingService benefitingService;
	@NotBlank
	@Length(max = 8)
	private String code;
	private boolean inactive;

	// -------------------------------------- Constructors

	// -------------------------------------- Business Methods

	// -------------------------------------- Common Methods

	@Override
	protected boolean requiredEquals(CombinationCode oo) {
		return new EqualsBuilder().append(nullSafeGetId(getStationProfile()), nullSafeGetId(oo.getStationProfile()))
				.append(nullSafeGetId(getOrganization()), nullSafeGetId(oo.getOrganization()))
				.append(nullSafeGetId(getSchedule()), nullSafeGetId(oo.getSchedule()))
				.append(nullSafeGetId(getBenefitingService()), nullSafeGetId(oo.getBenefitingService())).isEquals();
	}

	@Override
	protected int requiredHashCode() {
		return new HashCodeBuilder().append(nullSafeGetId(getStationProfile())).append(nullSafeGetId(getOrganization()))
				.append(nullSafeGetId(getSchedule())).append(nullSafeGetId(getBenefitingService())).toHashCode();
	}

	public String toString() {
		return getCode() + " (" + VolunteerStationProfile.class.getSimpleName() + " ID "
				+ nullSafeGetId(getStationProfile()) + ", " + Organization.class.getSimpleName() + " ID "
				+ nullSafeGetId(getOrganization()) + ", " + WorkdaySchedule.class.getSimpleName() + " ID "
				+ nullSafeGetId(getSchedule()) + ", " + BenefitingService.class.getSimpleName() + " ID "
				+ nullSafeGetId(getBenefitingService()) + ")";
	}

	// -------------------------------------- Accessor Methods

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "VolunteerStationProfilesFK", nullable = false)
	public VolunteerStationProfile getStationProfile() {
		return stationProfile;
	}

	public void setStationProfile(VolunteerStationProfile stationProfile) {
		this.stationProfile = stationProfile;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "OrganizationFK", nullable = false)
	public Organization getOrganization() {
		return organization;
	}

	public void setOrganization(Organization organization) {
		this.organization = organization;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "WorkdayScheduleFK", nullable = false)
	public WorkdaySchedule getSchedule() {
		return schedule;
	}

	public void setSchedule(WorkdaySchedule schedule) {
		this.schedule = schedule;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "BenefitingServiceFK", nullable = false)
	public BenefitingService getBenefitingService() {
		return benefitingService;
	}

	public void setBenefitingService(BenefitingService benefitingService) {
		this.benefitingService = benefitingService;
	}

	@Column(name = "CombinationCode", length = 8, nullable = false)
	public String getCode() {
		return code;
	}

	public void setCode(String code) {
		this.code = code;
	}

	@Column(name = "IsInactive", nullable = false)
	public boolean isInactive() {
		return inactive;
	}

	public void setInactive(boolean inactive) {
		this.inactive = inactive;
	}

}
