package gov.va.vss.model;

import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.NONE;
import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.PUBLIC_ONLY;

import java.time.ZonedDateTime;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import org.apache.commons.lang3.builder.CompareToBuilder;
import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;
import org.hibernate.annotations.BatchSize;
import org.springframework.format.annotation.DateTimeFormat;

import com.fasterxml.jackson.annotation.JsonAutoDetect;

import gov.va.shared.model.AppUser;
import gov.va.shared.model.core.AbstractAuditedVersionedPersistent;
import gov.va.vss.model.lookup.VtkInstitution;
import gov.va.vss.util.DateUtil;

@Entity
@Table(name = "VtkUsers", schema = "dbo")
@JsonAutoDetect(fieldVisibility = NONE, getterVisibility = PUBLIC_ONLY, isGetterVisibility = PUBLIC_ONLY)
public class VtkUser extends AbstractAuditedVersionedPersistent<VtkUser> implements Comparable<VtkUser> {

	private static final long serialVersionUID = 8168397789547901768L;
	// ------------------------------------ Fields

	private String ntUserName;
	private String firstName;
	private String middleName;
	private String lastName;
	private String nameSuffix;
	private String nickName;
	private String sex;
	private String grade;
	// private String lowercaseNameForDisplay;
	// private String uppercaseNameForDisplay;
	// private String fullNameForReports;
	private String emailAddress;
	private boolean isInactive = false;
	private String inactivator;
	@DateTimeFormat(pattern = DateUtil.TWO_DIGIT_DATE_ONLY)
	private ZonedDateTime lastAccess;
	private VtkInstitution primaryInstitution;
	private VtkInstitution defaultInstitution;
	private AppUser appUser;
	private VtkStaffTitle vtkStaffTitle;

	// ------------------------------------ Constructors

	public VtkUser() {
	}

	public VtkUser(AppUser u) {
		setFromAppUser(u);
	}

	// ------------------------------------ Business Methods

	public void setFromAppUser(AppUser u) {
		if (u != null) {
			setAppUser(u);
			setNtUserName(u.getUsername());
			setFirstName(u.getFirstName());
			setMiddleName(u.getMiddleName());
			setLastName(u.getLastName());
			setEmailAddress(u.getEmail());
		}
	}

	// ------------------------------------ Common Methods

	@Override
	protected boolean requiredEquals(VtkUser oo) {
		return new EqualsBuilder().append(ntUserName, oo.ntUserName).isEquals();
	}

	@Override
	protected int requiredHashCode() {
		return new HashCodeBuilder().append(ntUserName).toHashCode();
	}

	public int compareTo(VtkUser u) {
		if (equals(u))
			return 0;
		return new CompareToBuilder().append(getNtUserName(), u.getNtUserName()).toComparison() > 0 ? 1 : -1;
	}

	// ------------------------------------ Accessor Methods

	@Column(name = "NTUSERNAME")
	public String getNtUserName() {
		return ntUserName;
	}

	public void setNtUserName(String ntUserName) {
		this.ntUserName = ntUserName;
	}

	@Column(name = "FIRSTNAME", nullable = false)
	public String getFirstName() {
		return firstName;
	}

	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	@Column(name = "MIDDLENAME")
	public String getMiddleName() {
		return middleName;
	}

	public void setMiddleName(String middleName) {
		this.middleName = middleName;
	}

	@Column(name = "LASTNAME", nullable = false)
	public String getLastName() {
		return lastName;
	}

	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	@Column(name = "NAMESUFFIX")
	public String getNameSuffix() {
		return nameSuffix;
	}

	public void setNameSuffix(String nameSuffix) {
		this.nameSuffix = nameSuffix;
	}

	@Column(name = "NICKNAME")
	public String get    Name() {
		return nickName;
	}

	public void set    Name(String nickName) {
		this.nickName = nickName;
	}

	@Column(name = "SEX")
	public String getSex() {
		return sex;
	}

	public void setSex(String sex) {
		this.sex = sex;
	}

	@Column(name = "GRADE")
	public String getGrade() {
		return grade;
	}

	public void setGrade(String grade) {
		this.grade = grade;
	}

	@Column(name = "EMAILADDRESS")
	public String getEmailAddress() {
		return emailAddress;
	}

	public void setEmailAddress(String emailAddress) {
		this.emailAddress = emailAddress;
	}

	@Column(name = "INACTIVE")
	public boolean getIsInactive() {
		return isInactive;
	}

	public void setIsInactive(boolean isInactive) {
		this.isInactive = isInactive;
	}

	@Column(name = "INACTIVATOR")
	public String getInactivator() {
		return inactivator;
	}

	public void setInactivator(String inactivator) {
		this.inactivator = inactivator;
	}

	@Column(name = "LASTACCESS")
	public ZonedDateTime getLastAccess() {
		return lastAccess;
	}

	public void setLastAccess(ZonedDateTime lastAccess) {
		this.lastAccess = lastAccess;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "PRIMARYINSTITUTIONFK", nullable = false)
	@BatchSize(size = 500)
	public VtkInstitution getPrimaryInstitution() {
		return primaryInstitution;
	}

	public void setPrimaryInstitution(VtkInstitution primaryInstitution) {
		this.primaryInstitution = primaryInstitution;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "DEFAULTINSTITUTIONFK")
	@BatchSize(size = 500)
	public VtkInstitution getDefaultInstitution() {
		return defaultInstitution;
	}

	public void setDefaultInstitution(VtkInstitution defaultInstitution) {
		this.defaultInstitution = defaultInstitution;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "APP_USER_IDFK")
	@BatchSize(size = 500)
	public AppUser getAppUser() {
		return appUser;
	}

	public void setAppUser(AppUser appUser) {
		this.appUser = appUser;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "JOBTITLEFK")
	@BatchSize(size = 500)
	public VtkStaffTitle getVtkStaffTitle() {
		return vtkStaffTitle;
	}

	public void setVtkStaffTitle(VtkStaffTitle vtkStaffTitle) {
		this.vtkStaffTitle = vtkStaffTitle;
	}

}
