package gov.va.vss.model.lookup;

import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.NONE;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;

import org.apache.commons.lang3.builder.CompareToBuilder;
import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;
import org.hibernate.annotations.BatchSize;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.JsonProperty;

import gov.va.shared.model.core.AbstractAuditedVersionedPersistent;
import gov.va.shared.model.lookup.sds.VAFacility;

@Entity
@Table(name = "VtkAdministrativeUnits", schema = "dbo")
@Cache(usage = CacheConcurrencyStrategy.NONSTRICT_READ_WRITE)
@JsonAutoDetect(fieldVisibility = NONE, getterVisibility = NONE, isGetterVisibility = NONE)
public class VtkAdministrativeUnit extends AbstractAuditedVersionedPersistent<VtkAdministrativeUnit>
		implements Comparable<VtkAdministrativeUnit> {
	private static final long serialVersionUID = -8678395783438462990L;

	// ---------------------------------------- Constants

	public static final String VTK_TYPE_VISN = "VISN";

	// ---------------------------------------- Fields

	@NotNull
	private String name;
	private int number;
	@NotNull
	private String type;

	private VAFacility visnFacility;

	// ---------------------------------------- Constructors

	public VtkAdministrativeUnit() {
	}

	public VtkAdministrativeUnit(VAFacility facility) {
		if (!facility.isVISN())
			throw new IllegalArgumentException("Can't create a " + VtkAdministrativeUnit.class.getName() + " from a "
					+ VAFacility.class.getName() + " of type " + facility.getFacilityType().getCode());

		this.name = facility.getName();
		this.number = facility.getId().intValue();
		this.type = VTK_TYPE_VISN;
		this.visnFacility = facility;
	}

	// ---------------------------------------- Business Methods

	// ---------------------------------------- Common Methods

	@Override
	protected boolean requiredEquals(VtkAdministrativeUnit oo) {
		return new EqualsBuilder().append(number, oo.getNumber()).isEquals();
	}

	@Override
	protected int requiredHashCode() {
		return new HashCodeBuilder().append(number).toHashCode();
	}

	@Override
	public int compareTo(VtkAdministrativeUnit o) {
		if (equals(o))
			return 0;

		return new CompareToBuilder().append(name, o.getName()).toComparison() > 0 ? 1 : -1;
	}

	// ---------------------------------------- Accessor Methods

	@Column(name = "UnitName", length = 50, nullable = false)
	@JsonProperty
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	@Column(name = "UnitNumber", nullable = false)
	@JsonProperty
	public int getNumber() {
		return number;
	}

	public void setNumber(int number) {
		this.number = number;
	}

	@Column(name = "Type", length = 5, nullable = false)
	@JsonProperty
	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "SDS_INSTITUTION_ID")
	@BatchSize(size=500)
	public VAFacility getVisnFacility() {
		return visnFacility;
	}

	public void setVisnFacility(VAFacility visnFacility) {
		this.visnFacility = visnFacility;
	}

}
