package gov.va.vss.persistence.dao.jobExecution;

import java.time.ZonedDateTime;
import java.util.List;

import gov.va.shared.model.AppUser;
import gov.va.vss.model.jobExecution.JobExecutionResult;
import gov.va.vss.model.jobExecution.JobExecutionResultItem;
import gov.va.vss.model.lookup.JobExecutionType.JobExecutionTypeValue;
import gov.va.vss.persistence.dao.CustomizableSortedDAO;
import gov.va.vss.persistence.queryCustomization.QueryCustomization;

/**
 * @author barrycon
 * 
 */
public interface JobExecutionResultItemDAO extends CustomizableSortedDAO<JobExecutionResultItem> {

	/**
	 * Finds a List of JobExecutionResultItems given the specified search
	 * criteria.
	 * 
	 * @param referencePositionId
	 *            If specified, all returned results will have a
	 *            referencePosition with this ID
	 * @param customization
	 * @return See description
	 */
	List<JobExecutionResult> findByCriteria(Long referencePositionId, QueryCustomization... customization);

	/**
	 * Bulk updates all JobExecutionResultItems to nullify the referencePosition
	 * field when that position is being deleted
	 * 
	 * @param positionId
	 *            The position which is being deleted
	 * @return The number of rows changed
	 */
	int bulkUpdateForPositionDeletion(long positionId, AppUser currentUser);

	int deleteByCriteria(Long jobExecutionResultId, Long referencePatientId, Long referencePositionId,
			Long referenceTeamId, ZonedDateTime parentOlderThanDate, Long parentReferenceJobExecutionId,
			JobExecutionTypeValue parentJobExecutionTypeValue);

}
