/**
 * Adds and removes classes to a list of links to allow keyboard and jaws screen
 * reader accessibility
 * 
 * @param string
 *            menuBarId
 * @param int
 *            mouseOffDelay
 * 
 * @modifiedby jtorreno 10.12.2012
 */
function accessibleDropdownMenu(menuBarId, mouseOffDelay) {
	var dropDownElementId = menuBarId.replace('#', '');

	if (dropdown = document.getElementById(dropDownElementId)) {

		var listItems = dropdown.getElementsByTagName('li');

		for (var i = 0; i < listItems.length; i++) {
			listItems[i].onmouseover = function() {
				this.className = addClass(this);
			}
			listItems[i].onmouseout = function() {
				var that = this;
				setTimeout(function() {
					that.className = removeClass(that);
				}, mouseOffDelay);
				this.className = that.className;
			}

			var anchor = listItems[i].getElementsByTagName('a');
			anchor = anchor[0];

			anchor.onfocus = function() {
				tabOn(this.parentNode);
			}
			anchor.onblur = function() {
				tabOff(this.parentNode);
			}
		}

		addIndexAndIcon(menuBarId);
	}

	function tabOn(li) {
		if (li.nodeName == 'LI') {
			li.className = addClass(li);
			tabOn(li.parentNode.parentNode);
		}
	}

	function tabOff(li) {
		if (li.nodeName == 'LI') {
			li.className = removeClass(li);
			tabOff(li.parentNode.parentNode);
		}
	}

	function addClass(li) {
		return li.className + ' ' + 'hover';
	}

	function removeClass(li) {
		return li.className.replace('hover', "");
	}

	/**
	 * Adds z index to the menu popup to make it on top of other images on the
	 * screen. It also appends arrow icons for menus with children
	 * 
	 * @param menuBarId
	 * @createdby jtorreno 10.12.2012
	 */
	function addIndexAndIcon(menuBarId) {
		var downArrowIcon = "<div class='downMenuArrow'></div>";
		var rightArrowIcon = "<div class='rightMenuArrow'></div>";

		$(menuBarId + ' a').each(
				function() {
					var i = 100000;

					if ($(this).parent('li').children('ul').size() > 0) {
						i++;
						$(this).parent('li').not('ul ul li').children('a')
								.append(downArrowIcon);
						$(this).parent('li ul li').children('a').append(
								rightArrowIcon);
						// modified to be compatible with IE 9 since span had
						// the z-index is necessary to make menu on top of any
						// image on the screen
						$(this).parent('li').children('ul').css('z-index', i);
					}
				})
	}
}

/**
 * Changes position of menu popup If browser screen width is less than 1020 or
 * screen resolution is 1024, and the menu contains a submenu item then it
 * displays the popup on the left side so it be viewable.
 * 
 * This functionality will only look for the menu id specified on the parameter
 * and would not navigate the whole menu bar
 * 
 * @createdby jtorreno 11.30.2012
 */
function setSubMenuPopUpPosition(menuId) {
	if ($("#" + menuId + " ul").size() > 0) {
		if ($(document).width() <= 1150 || screen.width <= 1024) {
			$("#" + menuId + " ul").css('left', '-180px');
			$("#" + menuId + " ul").css('top', '22px');
			// $("#" + menuId + " ul").css('padding', '1px 1px 1px 1px');
		} else {
			$("#" + menuId + " ul").css('top', '12px');
			$("#" + menuId + " ul").css('left', '165px');
		}

		$("#" + menuId + " ul").css('z-index ', '9999999999');
		$("#" + menuId + " ul").css('position', 'absolute');
		$("#" + menuId + " ul").css('width', 'auto');
		$("#" + menuId + " ul").css('white-space', 'nowrap');
	}
}
