package gov.va.vss.persistence;

import java.time.LocalDate;
import java.util.List;

import org.junit.Assert;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;

import gov.va.vss.AbstractWebDAOTest;
import gov.va.vss.model.VoluntaryServiceStaff;
import gov.va.vss.model.VtkUser;
import gov.va.vss.model.lookup.VtkInstitution;
import gov.va.vss.persistence.dao.CustomizableAppDAO;
import gov.va.vss.persistence.dao.VoluntaryServiceStaffDAO;

public class TestVoluntaryServiceStaffDAO extends AbstractWebDAOTest<VoluntaryServiceStaff> {

	@Autowired
	private VoluntaryServiceStaffDAO voluntaryServiceStaffDAO;

	@Override
	protected CustomizableAppDAO<VoluntaryServiceStaff> getDAO() {
		return voluntaryServiceStaffDAO;
	}

	@Override
	protected VoluntaryServiceStaff getInstanceToSave() {
		VoluntaryServiceStaff v = new VoluntaryServiceStaff();
		v.setAppUser(user);
		v.setReportOrder(1);
		VtkInstitution vtkInstitution = findVtkInstitution("442");
		v.setVtkInstitution(vtkInstitution);
		VtkUser vtkUser = new VtkUser();
		vtkUser.setFirstName("Stacey");
		vtkUser.setLastName("Zhang");
		vtkUser.setPrimaryInstitution(vtkInstitution);
		vtkUser = vtkUserDAO.saveOrUpdate(vtkUser);
		v.setVtkUser(vtkUser);
		return v;
	}

	@Test
	public void testFindByCriteria() {
		List<VoluntaryServiceStaff> staffsFor442 = voluntaryServiceStaffDAO.findByCriteria("442", null,
				LocalDate.now());
		Assert.assertFalse("Failed - 442 didn't return any results", staffsFor442.isEmpty());

		staffsFor442 = voluntaryServiceStaffDAO.findByCriteria(null, "fakeAppUserName", LocalDate.now());
		Assert.assertTrue("Failed - 442 returned results for fake username", staffsFor442.isEmpty());
	}

	protected VtkInstitution findVtkInstitution(String stationNum) {
		return vtkInstitutionDAO.findByStationNumber(stationNum);
	}

	@Override
	protected boolean testDelete() {
		return true;
	}

}
