package gov.va.vss.model;

import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.NONE;
import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.PUBLIC_ONLY;

import java.time.LocalDate;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;
import org.springframework.format.annotation.DateTimeFormat;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.JsonIgnore;

import gov.va.shared.model.AppUser;
import gov.va.shared.model.core.AbstractAuditedVersionedPersistent;
import gov.va.vss.model.lookup.VtkInstitution;
import gov.va.vss.util.DateUtil;

@Entity
@Table(name = "VtkVoluntaryServiceStaff", schema = "dbo")
@JsonAutoDetect(fieldVisibility = NONE, getterVisibility = PUBLIC_ONLY, isGetterVisibility = PUBLIC_ONLY)
public class VoluntaryServiceStaff extends AbstractAuditedVersionedPersistent<VoluntaryServiceStaff> {
	private static final long serialVersionUID = -838492419896709672L;

	// -------------------------------------- Fields

	private VtkInstitution vtkInstitution;
	private Integer reportOrder = 0; // reportOrder is required, but there is no
										// input for it. assign zero as default
	private String comment;
	private String namePrefix;
	private String nickName;
	private String grade;
	private LocalDate vavsStartDate;
	private LocalDate vavsEndDate;
	private LocalDate retirementEstimateDate;
	private LocalDate retirementEligibleDate;
	private AppUser appUser;
	private VtkStaffTitle staffTitle;
	private boolean chief;
	private boolean vavsLeadership;
	private VtkUser vtkUser;

	// -------------------------------------- Constructors

	public VoluntaryServiceStaff() {
	}

	public VoluntaryServiceStaff(VtkInstitution vtkInstitution) {
		this.vtkInstitution = vtkInstitution;
	}
	
	// -------------------------------------- Business Methods

	@Transient
	public BasicUserFields getUserInfo() {
		return appUser != null ? new BasicUserFields.AppUserAdapter(appUser) : vtkUser;
	}

	// ---------------------------------------- Common Methods

	@Override
	protected boolean requiredEquals(VoluntaryServiceStaff oo) {
		return new EqualsBuilder().append(nullSafeGetId(getAppUser()), nullSafeGetId(oo.getAppUser()))
				.append(nullSafeGetId(getVtkInstitution()), nullSafeGetId(oo.getVtkInstitution())).isEquals();
	}

	@Override
	protected int requiredHashCode() {
		return new HashCodeBuilder().append(nullSafeGetId(getAppUser())).append(nullSafeGetId(getVtkInstitution()))
				.toHashCode();
	}

	// -------------------------------------- Accessor Methods

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "InstitutionFK", unique = true)
	@JsonIgnore
	public VtkInstitution getVtkInstitution() {
		return vtkInstitution;
	}

	public void setVtkInstitution(VtkInstitution vtkInstitution) {
		this.vtkInstitution = vtkInstitution;
	}

	@Column(name = "REPORTORDER", nullable = false)
	public Integer getReportOrder() {
		return reportOrder;
	}

	public void setReportOrder(Integer reportOrder) {
		this.reportOrder = reportOrder;
	}

	@Column(name = "COMMENT")
	public String getComment() {
		return comment;
	}

	public void setComment(String comment) {
		this.comment = comment;
	}

	@Column(name = "NAMEPREFIX")
	public String getNamePrefix() {
		return namePrefix;
	}

	public void setNamePrefix(String namePrefix) {
		this.namePrefix = namePrefix;
	}

	@Column(name = "NICKNAME")
	public String get    Name() {
		return nickName;
	}

	public void set    Name(String nickName) {
		this.nickName = nickName;
	}

	@Column(name = "GRADE")
	public String getGrade() {
		return grade;
	}

	public void setGrade(String grade) {
		this.grade = grade;
	}

	@Column(name = "VAVSSTARTDATE")
	@DateTimeFormat(pattern = DateUtil.TWO_DIGIT_DATE_ONLY)
	public LocalDate getVavsStartDate() {
		return vavsStartDate;
	}

	public void setVavsStartDate(LocalDate vavsStartDate) {
		this.vavsStartDate = vavsStartDate;
	}

	@Column(name = "VAVSENDDATE")
	@DateTimeFormat(pattern = DateUtil.TWO_DIGIT_DATE_ONLY)
	public LocalDate getVavsEndDate() {
		return vavsEndDate;
	}

	public void setVavsEndDate(LocalDate vavsEndDate) {
		this.vavsEndDate = vavsEndDate;
	}

	@Column(name = "RETIREMENTESTIMATEDATE")
	@DateTimeFormat(pattern = DateUtil.TWO_DIGIT_DATE_ONLY)
	public LocalDate getRetirementEstimateDate() {
		return retirementEstimateDate;
	}

	public void setRetirementEstimateDate(LocalDate retirementEstimateDate) {
		this.retirementEstimateDate = retirementEstimateDate;
	}

	@Column(name = "RETIREMENTELIGIBLEDATE")
	@DateTimeFormat(pattern = DateUtil.TWO_DIGIT_DATE_ONLY)
	public LocalDate getRetirementEligibleDate() {
		return retirementEligibleDate;
	}

	public void setRetirementEligibleDate(LocalDate retirementEligibleDate) {
		this.retirementEligibleDate = retirementEligibleDate;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "APP_USER_IDFK", unique = true, nullable = false)
	public AppUser getAppUser() {
		return appUser;
	}

	public void setAppUser(AppUser appUser) {
		this.appUser = appUser;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "VTKSTAFFTITLESFK")
	public VtkStaffTitle getStaffTitle() {
		return staffTitle;
	}

	public void setStaffTitle(VtkStaffTitle staffTitle) {
		this.staffTitle = staffTitle;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "USERFK", unique = true)
	@JsonIgnore
	public VtkUser getVtkUser() {
		return vtkUser;
	}

	public void setVtkUser(VtkUser vtkUser) {
		this.vtkUser = vtkUser;
	}

	@Column(name = "ISCHIEF")
	public boolean isChief() {
		return chief;
	}

	public void setChief(boolean chief) {
		this.chief = chief;
	}

	@Column(name = "ISVISNLIAISON")
	public boolean isVavsLeadership() {
		return vavsLeadership;
	}

	public void setVavsLeadership(boolean vavsLeadership) {
		this.vavsLeadership = vavsLeadership;
	}

}
