<%@ include file="../shared/inc_header.jsp"%>

<%--
	Expects the following params:
	
	- A "uniqueVolunteerSearchPopupId" param which should be a string of letters that
		uniquely identifies this volunteerSearchPopup among other volunteerSearchPopups on the same page
	- A "callbackMethod" param which specifies the name of a JavaScript method defined by the
		-including- page which the volunteer popup will activate when the user selects a volunteer. The
		required method signature looks like:
		function someMethodWithUniqueName(volunteerObj)
		
		The volunteerObj parameter sent to the callback method is a javascript object containing the
		properties
		{
			id : <the ID of the volunteer selected>,
			[... other attributes of the Volunteer class marked for JsonProperty]
		}
		
	To activate this popup, call the method popupVolunteerSearch(uid) and pass in the same
	uniqueVolunteerSearchPopupId String above.
--%>

<%-- Escaping XML here for good practice since we use the raw EL expressions later --%>
<c:set var="uid"
	value="${fn:escapeXml(param.uniqueVolunteerSearchPopupId)}" />
<c:set var="callbackMethod"
	value="${fn:escapeXml(param.resultCallbackMethod)}" />

<script type="text/javascript">
function volunteerSearchPopupItemSelected(uid, volunteerId) {
	var volunteerObj = volunteerSearchResults[uid]['' + volunteerId]
	$("#volunteerSearchDialog" + uid).dialog('close')
	${callbackMethod}(volunteerObj)
}

$(function() {
	initVolunteerSearchPopup('${uid}')
})
</script>

<div id="volunteerSearchDialog${uid}" style="display: none"
	title="Search for Volunteer">
	<table align="center">
		<tr valign="middle">
			<td align="right"><label for='volunteerSearchName${uid}'>Name:</label></td>
			<td><input type="text" id="volunteerSearchName${uid}" value=""
				title="Type a name in the format Last[, First Middle]"
				placeholder="Last[, First Middle]" /></td>
			<td width="20" rowspan="3">&nbsp;</td>
			<td align="right" class="advSearchFields${uid}"><label
				for='volunteerSearchName${uid}'>Identifying Code:</label></td>
			<td class="advSearchFields${uid}"><input type="text"
				id="volunteerSearchCode${uid}" value=""
				title="Type an identifying code" /></td>

			<td width="20" rowspan="3">&nbsp;</td>
			<td class="advSearchFields${uid}"><input type="radio"
				id="volunteerSearchScope${uid}Local" name="volunteerSearchScope${uid}"
				value="Local" checked="checked">Within <c:out
					value="${siteContext.sta3n}" /></td>
			<td class="simpleSearchFields${uid}"><a
				class="buttonAnchor volunteerSearchLink${uid}" tabIndex="0">Search</a></td>
			<td align="right" width="240" nowrap id="advancedSearchLink${uid}"
				class="simpleSearchFields${uid}"><a class="appLink"
				href="javascript:volunteerAdvancedSearch('${uid}', true)">[Advanced
					Search]</a></td>
		</tr>
		<tr class="advSearchFields${uid}">
			<td align="right"><label for='volunteerSearchAddress${uid}'>Address:</label></td>
			<td><input type="text" id="volunteerSearchAddress${uid}"
				value="" title="Type an address" /></td>
			<td align="right"><label for='volunteerSearchLocation${uid}'>Location:</label></td>
			<td><input type="text" id="volunteerSearchLocation${uid}"
				value=""
				title="Type a location in the format City[, State] or [Zip]"
				placeholder="City[, State] or Zip" /></td>
			<td class="advSearchFields${uid}"><input type="radio"
				id="volunteerSearchScope${uid}National" name="volunteerSearchScope${uid}"
				value="national">National</td>
		</tr>
		<tr class="advSearchFields${uid}">
			<td align="right"><label for='volunteerSearchPhone${uid}'>Phone:</label></td>
			<td><input type="text" id="volunteerSearchPhone${uid}" value=""
				title="Type a phone number" /></td>
			<td align="right"><label for='volunteerSearchEmail${uid}'>Email:</label></td>
			<td><input type="text" id="volunteerSearchEmail${uid}" value=""
				title="Type all or part of an email" /></td>
			<td class="advSearchFields"><a style="margin-left: 20px"
				class="buttonAnchor volunteerSearchLink${uid}" tabIndex="0">Search</a></td>
		</tr>
	</table>
	<div id="volunteerSearchNoResults${uid}" style="display: none">
		<table align="center" cellpadding="10">
			<tr>
				<td>Sorry, no volunteers were found that matched the specified
					criteria.</td>
			</tr>
		</table>
	</div>
	<div id="volunteerSearchResultsTable${uid}" style="display: none">
		<table summary="Format table" align="center" width="98%">
			<tr>
				<td align="center">
					<table class="formatTable" id="volunteerSearchResultsList${uid}"
						border="1" summary="List of Volunteers" width="100%">
						<thead>
							<tr id="volunteerSearch${uid}FilterRow">
								<td class="noborder">Filters:</td>
								<td class="noborder"></td>
								<td class="noborder " title="Filter by Gender"></td>
								<td class="noborder"></td>
								<td class="noborder"></td>
							</tr>
							<tr>
								<th width="40%" id="nameHeaderCol${uid}">Name</th>
								<th width="10%">Date of Birth</th>
								<th width="10%" class="select-filter">Gender</th>
								<th width="20%">Contact Info</th>
								<th width="10%" nowrap>Identifying Code</th>
							</tr>
						</thead>
					</table>
				</td>
			</tr>
		</table>
	</div>
</div>
