/* 
	widget has structure like:
		
	initialize(initCompleteCallback) = function to initialize this widget's data
	getReportParameters() - function which returns an Object whose keys/values are parameter
		keys/values we want to pass into the report. items having multiple values
		can be separated with commas or however the report needs to process them.
	getNonReportParameters() - an OPTIONAL function which returns an Object whose keys/values
		can be retrieved and used in report parameter javascript functions, as needed, but
		which are not included when sending the request to generate the report.
 */
function Widget(config) {
	if (!config) {
		alert('Missing required config param in widget constructor')
		return
	}
	
	this.id = config.id
	if (!this.id) {
		alert('Missing required config attribute "id" in widget constructor ')
		return
	}
	
	this.initialize = config.initialize
	if (!this.initialize) {
		/* No initialize provided; default implementation just returns - CPB */
		this.initialize = function(initCompleteCallback) {
			initCompleteCallback()
		}
	} else {	
		if (!$.isFunction(this.initialize) || this.initialize.length < 1) {
			alert('Missing required config function initialize(initCompleteCallback) in widget with ID "' + this.id + '"')
			return
		}
	}
	
	this.getParameters = config.getParameters
	if (!this.getParameters || !$.isFunction(this.getParameters)) {
		alert('Missing required config attribute getParameters() in widget with ID "' + this.id + '"')
	}
	
	this.valueUpdatedListeners = []
	this.changeEventSelectors = config.changeEventSelectors || []
	this.submitAsReportParam = config.submitAsReportParam
	this.bypassAsReportParam = config.bypassAsReportParam
	this.visible = config.visible
}

Widget.prototype.show = function() {
	$("#widget_" + this.id + "_div").show()
	this.visible = true
}

Widget.prototype.hide = function() {
	$("#widget_" + this.id + "_div").hide()
	this.visible = false
}

Widget.prototype.toggle = function(theVal) {
	$("#widget_" + this.id + "_div").toggle(theVal)
	this.visible = theVal
}

Widget.prototype.registerChangeEventSelectors = function() {
	var that = this
	$.each(this.changeEventSelectors, function(index, item) {
		$(item).change(function(evt) {
			that.valueChanged(evt)
		})
	})
}

Widget.prototype.addValueUpdatedListener = function(callback) {
	if (!callback || !$.isFunction(callback)) {
		alert('Callback must be provided and have a type of function')
		return
	}
	
	this.valueUpdatedListeners.push(callback)
}

Widget.prototype.valueChanged = function(evt) {
	var that = this
	$.each(this.valueUpdatedListeners, function(index, callback) {
		callback(evt, that.getParameters(), that)
	})
}

/*
 * Return the value of the report parameter with the specified paramName, or the
 * first parameter value if paramName is unspecified (for convenience) - CPB
 */
Widget.prototype.getParameterValue = function(paramName) {
	var params = this.getParameters()
	for (var i = 0; i < params.length; i++)
		if (!paramName || params[i].paramName == paramName)
			return params[i].paramValue
	return null
}

Widget.prototype.bypassReportParameters = function(theVal) {
	if (typeof theVal != 'undefined') {
		this.bypassAsReportParam = theVal
	} else {
		this.bypassAsReportParam = true
	}
}