package gov.va.vss.persistence;

import java.time.ZonedDateTime;

import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallback;
import org.springframework.transaction.support.TransactionTemplate;

import gov.va.shared.model.lookup.ActiveStatus;
import gov.va.shared.model.lookup.ActiveStatus.ActiveStatusType;
import gov.va.shared.model.lookup.LookupType;
import gov.va.vss.persistence.dao.CustomizableLookupDAO;
import gov.va.vss.persistence.dao.lookup.ActiveStatusDAO;
import gov.va.vss.util.DateUtil;

public class TestActiveStatusDAO extends AbstractLookupDAOTest<ActiveStatus> {

	@Autowired
	private ActiveStatusDAO activeStatusDAO;

	@Override
	protected ActiveStatus getInstanceToSave() {
		ActiveStatus p = new ActiveStatus();
		p.setEffectiveDate(ZonedDateTime.of(2015, 1, 1, 0, 0, 0, 0, DateUtil.UTC));
		p.setId(4L);
		p.setName("TestStatus");
		p.setDescription("TestDesc");
		p.setSortOrder(5);
		return p;
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	@Test
	public void testSpringCache() {
		for (int i = 0; i < 10; i++) {
			new TransactionTemplate(tm).execute(new TransactionCallback() {
				public Object doInTransaction(TransactionStatus ts) {
					activeStatusDAO.findByLookup(ActiveStatusType.ACTIVE);
					return null;
				}
			});
		}
	}

	@Override
	protected CustomizableLookupDAO<ActiveStatus> getLookupDAO() {
		return activeStatusDAO;
	}

	@Override
	protected LookupType getExampleLookupType() {
		return ActiveStatusType.ACTIVE;
	}
}