package gov.va.vss.persistence;

import java.util.ArrayList;
import java.util.List;

import org.junit.Assert;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;

import gov.va.vss.AbstractTransactionalWebDAOTest;
import gov.va.vss.model.Organization;
import gov.va.vss.model.OrganizationScopeType;
import gov.va.vss.model.lookup.VtkOrganizationType.VtkOrganizationTypeValue;
import gov.va.vss.persistence.dao.CustomizableAppDAO;
import gov.va.vss.persistence.dao.OrganizationDAO;

public class TestOrganizationDAO extends AbstractTransactionalWebDAOTest<Organization> {

	@Override
	protected CustomizableAppDAO<Organization> getDAO() {
		return organizationDAO;
	}

	@Override
	protected Organization getInstanceToSave() {
		Organization o = new Organization();
		o.setName("UnitTestOrganization");
		o.setFullName(o.getName());
		o.setType(vtkOrganizationTypeDAO.findByLookup(VtkOrganizationTypeValue.ACADEMIC));
		o.setCode(99999);
		o.setScope(OrganizationScopeType.LOCAL);
		return o;
	}

	@Override
	protected boolean testDelete() {
		return true;
	}

	@Test
	public void testFindByCriteria() {
		List<Long> staList1 = new ArrayList<Long>();
		staList1.add(1234L);
		organizationDAO.findByCriteria("test", false, staList1, false);
		organizationDAO.findByCriteria("test", false, staList1, true);
		organizationDAO.findByCriteria("test", true, staList1, false);
		organizationDAO.findByCriteria("test", true, staList1, true);

		List<Long> staList2 = new ArrayList<Long>();
		staList2.add(1000098L);
		List<Organization> resultsFor442 = organizationDAO.findByCriteria(null, true, staList2, true);
		Assert.assertFalse("Failed - 442 didn't return any local or national results", resultsFor442.isEmpty());
		resultsFor442 = organizationDAO.findByCriteria(null, true, staList2, false);
		Assert.assertFalse("Failed - 442 didn't return any local results", resultsFor442.isEmpty());



	}

	@Test
	public void testFindNextAvailableCode() {
		organizationDAO.findNextAvailableCode(OrganizationScopeType.NATIONAL, null);
		organizationDAO.findNextAvailableCode(OrganizationScopeType.LOCAL, 1000098L);
		organizationDAO.findNextAvailableCode(OrganizationScopeType.LOCAL, -1L);
		
		//CCR 275208  Local code should be in the range 900-9999
		try {
			// test that the local code max out at 9999
			organizationDAO.findNextAvailableCode(OrganizationScopeType.LOCAL, 1000212L);
			Assert.assertFalse("Failed - facility 623 should not return any local results", true);
		} catch (AssertionError e ) {
			// expecting exception here
		}
		
		// test that the local code starts from 900
		// station FK=281, 565GC, SDS ID=1001126
		int nextCode = organizationDAO.findNextAvailableCode(OrganizationScopeType.LOCAL, 1001126L);
		Assert.assertEquals(900, nextCode);
	}

}
