package gov.va.vss.model.timeline;

import java.time.ZonedDateTime;

import javax.persistence.Column;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.MappedSuperclass;
import javax.validation.constraints.NotNull;

import org.apache.commons.lang.builder.CompareToBuilder;

import com.fasterxml.jackson.annotation.JsonProperty;

import gov.va.shared.model.core.AbstractAuditedVersionedPersistent;
import gov.va.shared.model.lookup.ActiveStatus;
import gov.va.shared.model.timeline.TimelineEventWithStatus;

@SuppressWarnings("rawtypes")
@MappedSuperclass
public abstract class AbstractTimelineEvent<T> extends AbstractAuditedVersionedPersistent<T>
		implements TimelineEventWithStatus, Comparable<AbstractTimelineEvent> {
	private static final long serialVersionUID = -3627940075643892637L;

	// ---------------------------------------- Fields

	@NotNull
	private ActiveStatus status;

	@NotNull
	private ZonedDateTime effectiveDate;

	// ---------------------------------------- Business Methods

	@Override
	public String toString() {
		return getEffectiveDate() + " - " + getStatus();
	}

	// ---------------------------------------- Common Methods

	@SuppressWarnings("unchecked")
	@Override
	public final int compareTo(AbstractTimelineEvent o) {
		if (equals(o))
			return 0;

		CompareToBuilder ctb = new CompareToBuilder().append(getEffectiveDate(), o.getEffectiveDate());
		appendExtraCompareToFields(ctb, (T) o);
		return ctb.toComparison() > 0 ? 1 : -1;
	}

	protected abstract void appendExtraCompareToFields(CompareToBuilder ctb, T o);

	// ---------------------------------------- Accessor Methods

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "PCM_STD_STATUS_ID")
	@JsonProperty
	public ActiveStatus getStatus() {
		return status;
	}

	public void setStatus(ActiveStatus assignmentStatus) {
		this.status = assignmentStatus;
	}

	@Column(name = "EFFECTIVE_DATE")
	@JsonProperty
	public ZonedDateTime getEffectiveDate() {
		return effectiveDate;
	}

	public void setEffectiveDate(ZonedDateTime effectiveDate) {
		this.effectiveDate = effectiveDate;
	}

}
