package gov.va.vss.service;

import java.time.ZonedDateTime;
import java.util.Collection;
import java.util.List;
import java.util.concurrent.TimeoutException;

import gov.va.vss.model.VistaStationData;

public interface VistaStationDataService {

	VistaStationData getOrCreateByStation(String stationNumber);

	/**
	 * Retrieves the VistaStationData with the specified stationNumber (creating
	 * a new one if necessary) and updates only the specified (non-null) fields
	 * 
	 * @throws InterruptedException
	 * @throws TimeoutException
	 */
	int saveOrUpdateWithFieldsInNewTransaction(String stationNumber, ZonedDateTime cprsHeaderSyncLastExecutionDate,
			ZonedDateTime patientInactivationLastExecutionDate, ZonedDateTime vistaCleanupProcessLastExecutionDate,
			ZonedDateTime staffUpdateLastExecutionDate);

	int setProcessesEnabled(String stationNumber, Boolean cprsHeaderSyncProcessEnabled,
			Boolean patientInactivationProcessEnabled, Boolean vistaSyncProcessEnabled,
			Boolean vistaCleanupProcessEnabled, Boolean staffUpdateProcessEnabled);

	List<VistaStationData> getStationsNeedingAutoInactivationProcessing();

	int updateCPRSHeaderSyncLastExecutionDatesForEnabledStations(Collection<String> stationNumbers,
			ZonedDateTime executionDate);

}
