package gov.va.vss.service.impl;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import gov.va.shared.model.AppUser;
import gov.va.shared.model.Permission.PermissionType;
import gov.va.shared.util.SecurityUtil;
import gov.va.vss.model.Organization;
import gov.va.vss.model.ScopeType;
import gov.va.vss.service.OrganizationService;
import gov.va.vss.service.validation.ServiceValidationException;

@Service
public class OrganizationServiceImpl extends AbstractServiceImpl implements OrganizationService {
	private static final Logger log = LoggerFactory.getLogger(OrganizationServiceImpl.class);

	@Override
	public Organization saveOrUpdate(Organization o, AppUser user, long vaFacilityId, boolean isEdit)
			throws ServiceValidationException {

		/* Business-level validations */
		if (o.getScope() == ScopeType.LOCAL
				&& !SecurityUtil.hasAllPermissionsAtFacility(vaFacilityId, PermissionType.ORG_CODE_LOCAL_CREATE))
			throw new ServiceValidationException("scope", "organization.saveOrUpdate.error");
		if (o.getScope() == ScopeType.NATIONAL
				&& !SecurityUtil.hasAllPermissionsAtFacility(vaFacilityId, PermissionType.ORG_CODE_NATIONAL_CREATE))
			throw new ServiceValidationException("scope", "organization.saveOrUpdate.error");

		if (!isEdit) {
			int nextCode = organizationDAO.findNextAvailableCode(o.getScope(), (o.getScope() == ScopeType.LOCAL? vaFacilityId: null));
			o.setCode(nextCode);
		}

		o = organizationDAO.saveOrUpdate(o);
		return o;
	}

}
