package gov.va.vss.service.impl;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import gov.va.vss.model.VoluntaryServiceStaff;
import gov.va.vss.model.lookup.VtkInstitution;
import gov.va.vss.service.AppUserAndVtkUser;
import gov.va.vss.service.VoluntaryServiceStaffService;
import gov.va.vss.service.VtkService;
import gov.va.vss.service.VtkUserService;

@Service
public class VoluntaryServiceStaffServiceImpl extends AbstractServiceImpl implements VoluntaryServiceStaffService {
	private static final Logger log = LoggerFactory.getLogger(VoluntaryServiceStaffServiceImpl.class);

	@Autowired
	private VtkService vtkService;
	@Autowired
	private VtkUserService vtkUserService;

	@Override
	public VoluntaryServiceStaff saveOrUpdate(VoluntaryServiceStaff serviceStaff) {
		return voluntaryServiceStaffDAO.saveOrUpdate(serviceStaff);
	}

	@Override
	public void delete(long serviceStaffId) {
		voluntaryServiceStaffDAO.delete(serviceStaffId);
	}

	@Override
	public VoluntaryServiceStaff createOrRetrieveServiceStaff(String staffAppUserName, long facilityId) {
		VtkInstitution institution = vtkService.getOrCreateInstitutionForSDSFacility(facilityId);
		
		List<VoluntaryServiceStaff> results = voluntaryServiceStaffDAO.findByCriteria(institution.getSta3n(), staffAppUserName, null);
		if (!results.isEmpty())
			return results.get(0);

		VoluntaryServiceStaff serviceStaff = new VoluntaryServiceStaff();
		AppUserAndVtkUser bothUsers = vtkUserService.createOrRetrieveAppUser(staffAppUserName);
		serviceStaff.setAppUser(bothUsers.appUser);
		serviceStaff.setVtkUser(bothUsers.vtkUser);

		
		serviceStaff.setVtkInstitution(institution);

		serviceStaff = saveOrUpdate(serviceStaff);
		return serviceStaff;
	}

}
