package gov.va.vss.service.scheduledJobs;

import java.time.ZonedDateTime;
import java.util.concurrent.locks.Lock;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.Profile;
import org.springframework.jmx.export.annotation.ManagedOperation;
import org.springframework.jmx.export.annotation.ManagedResource;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import gov.va.vss.model.jobExecution.JobExecutionResult;
import gov.va.vss.model.jobExecution.JobExecutionResultItem;
import gov.va.vss.model.lookup.JobExecutionType;
import gov.va.vss.model.lookup.JobExecutionType.JobExecutionTypeValue;

@Service
@Profile(value = { "default", "unattended" })
@ManagedResource
public class ExcludedEntityRefreshJob extends AbstractScheduledJob {
	private static final Logger log = LoggerFactory.getLogger(ExcludedEntityRefreshJob.class);

	@Scheduled(cron = "${scheduledJobs.excludedEntityRefresh.cron}")
	// or if it's on a delay, use this config:
	// @Scheduled(initialDelayString = "${scheduledJobs.startupDelayMillis}", //
	// fixedDelayString = "${scheduledJobs.cprsHeaderSync.fixedDelay}")
	@ManagedOperation
	public void refreshExcludedEntities() {
		log.info(Thread.currentThread().toString() + ": Running scheduled job {}",
				ExcludedEntityRefreshJob.class.getSimpleName());

		String lockName = getClass().getName();
		Lock lock = clusterOpsUnattendedOnly.getLock(lockName);
		if (!lock.tryLock())
			return;

		JobExecutionType type = jobExecutionTypeDAO.findByLookup(JobExecutionTypeValue.LEIE);
		ZonedDateTime now = ZonedDateTime.now();
		JobExecutionResult result = new JobExecutionResult(type, null, now);
		result = jobExecutionResultService.saveOrUpdate(result);

		JobExecutionResultItem item = jobExecutionResultItemService.saveItemResult(result, now, null, false,
				"Refreshing excluded entities...", null);

		try {
			int results = excludedEntityService.refreshData();
			item.setItemSuccessful(true);
			item.setItemStatusMessageStr(
					item.getItemStatusMessageStr() + " " + results + " excluded entities processed.");
		} catch (Exception t) {
			log.error("Caught exception in " + ExcludedEntityRefreshJob.class.getName(), t);
			item.setStatusException(t);
		} finally {
			try {
				item.setEndTime(ZonedDateTime.now());
				jobExecutionResultItemService.saveOrUpdate(item);
			} catch (Exception ignored) {
			}
			lock.unlock();
		}
	}
}
