package gov.va.vss.web.reports;

import java.util.Arrays;
import java.util.Enumeration;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;

import gov.va.vss.web.AbstractAppController;
import gov.va.vss.web.CommonController;
import gov.va.vss.web.interceptor.BreadcrumbsInterceptor;

@Controller
@SessionAttributes(value = { AbstractAppController.DEFAULT_COMMAND_NAME })
public class CannedReportsController extends AbstractAppController {

	private static final String PROP_REPORT_PREFIX = "reports.";

	@Value("${ssrs.viewReport.baseURL}")
	private String baseURL;

	/** TODO: Fetch from env specific property file **/

	private static final String reportRenderCmd = "&rs:Command=Render";

	@RequestMapping("/displayReportParameters")
	public String displayReportParameters(@RequestParam(required = true) String reportCode, ModelMap model,
			HttpServletRequest request) {
		model.put("reportBaseURL", baseURL);
		model.put("allStates", vtkStateDAO.findAllSorted());
		
		String displayName = env.getRequiredProperty(PROP_REPORT_PREFIX + reportCode + ".displayName");
		model.put("pageTitle", "Specify Parameters for the " + displayName + " Report");
		model.put("pageDescription",
				"This screen allows the user to specify parameters for the " + displayName + " Report");
		model.put("ssrsReportBasename",
				env.getRequiredProperty(PROP_REPORT_PREFIX + reportCode + ".ssrsReportBasename"));
		model.put("disclaimerText", env.getProperty(PROP_REPORT_PREFIX + reportCode + ".disclaimerText"));
		BreadcrumbsInterceptor.setRequestBreadcrumb(request, "Run Report \"" + displayName + "\"");

		return env.getRequiredProperty(PROP_REPORT_PREFIX + reportCode + ".tilesView");
	}

	@RequestMapping("/executeReport")
	public String executeCannedReport(@RequestParam(required = true) String reportName, ModelMap model,
			HttpServletRequest request) {
		model.put("reportURL", mapReportURL(reportName, request));
		return "streamReport";
	}

	public String mapReportURL(String reportName, HttpServletRequest request) {
		/*
		 * TODO: Move the mapping of location paths to the reports (based on
		 * passed in reportName) into a metadata table in the DB. Currently all
		 * reports stored in /
		 **/
		String reportFolderPath = "/";

		String reportURL = baseURL + reportFolderPath + reportName + reportRenderCmd;
		// + (request.getQueryString() != null ? ("&" +
		// request.getQueryString()) : "");
		Enumeration<String> parameterNames = request.getParameterNames();
		while (parameterNames.hasMoreElements()) {
			String parameterName = parameterNames.nextElement();
			if (parameterName.equalsIgnoreCase("reportName")) {
				continue;
			} else {
				reportURL = reportURL + "&" + parameterName + "=" + request.getParameter(parameterName);
			}
		}

		return reportURL;
	}

	private List<Integer> generateBoxedConsecutiveIntegerStream(int low, int high) {
		List<Integer> numbersList = IntStream.rangeClosed(low, high).boxed().collect(Collectors.toList());
		return numbersList;
	}

	private void createReferenceData(ModelMap model, String reportName) {
		if (reportName != null && reportName.equalsIgnoreCase("queryBased_AddressLabelsReport")) {
			// Populate report specific parameter lists
			model.put("allSearchPaths", Arrays.asList(SearchPathCommand.values()));
			CommonController.populateFacilityList(getCurrentUser(), model, vaFacilityDAO);

			// vtkInstitutionDAO.findAllSorted();

			model.put("agesParamList", generateBoxedConsecutiveIntegerStream(1, 150));
			model.put("monthsParamList", generateBoxedConsecutiveIntegerStream(1, 12));
		}
	}

}
