package gov.va.shared.config;

import java.io.File;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.MessageSource;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.PropertySource;
import org.springframework.context.annotation.PropertySources;
import org.springframework.context.support.PropertySourcesPlaceholderConfigurer;
import org.springframework.context.support.ReloadableResourceBundleMessageSource;
import org.springframework.core.env.Environment;
import org.springframework.scheduling.annotation.AsyncAnnotationBeanPostProcessor;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;

@PropertySource("file:///${AppPropertiesDir:" + AbstractConfig.DEFAULT_APP_PROPERTIES_DIR + "}/app.properties")
@PropertySource("file:///${AppPropertiesDir:" + AbstractConfig.DEFAULT_APP_PROPERTIES_DIR + "}/version.properties")
public abstract class AbstractConfig {

	private static final String DEFAULT_BASEDIR = "C:/data/projects-hp/VSS/trunk";
	public static final String DEFAULT_APP_PROPERTIES_DIR = DEFAULT_BASEDIR + "/properties/local";
	public static final String DEFAULT_MESSAGES_DIR = DEFAULT_BASEDIR + "/web/src/main/resources";

	@Autowired
	protected Environment env;

	@Value("${AppPropertiesDir:" + DEFAULT_MESSAGES_DIR + "}")
	private String messagesDir;
	@Value("${AppPropertiesDir:" + DEFAULT_APP_PROPERTIES_DIR + "}")
	private String propertiesDir;

	public static boolean isUnitTest(Environment env) {
		return env.acceptsProfiles("default");
	}

	@Bean
	/* Has to be static per Spring's bootstrap process - CPB */
	public static PropertySourcesPlaceholderConfigurer placeHolderConfigurer() {
		return new PropertySourcesPlaceholderConfigurer();
	}

	@Bean
	public MessageSource messageSource() {
		ReloadableResourceBundleMessageSource ms = new ReloadableResourceBundleMessageSource();
		ms.setCacheSeconds(Integer.parseInt(env.getProperty("messages.cache.expirySeconds")));
		ms.setBasenames("file:///" + messagesDir + File.separator + "messages",
				"file:///" + propertiesDir + File.separator + "app");
		return ms;
	}

	@Bean
	public AsyncAnnotationBeanPostProcessor asyncPostProcessor() {
		return new AsyncAnnotationBeanPostProcessor();
	}

	// @Bean
	// public Properties appProperties() throws IOException {
	// PropertiesFactoryBean propertiesFactoryBean = new
	// PropertiesFactoryBean();
	// propertiesFactoryBean.setLocation(new FileSystemResource(
	// env.getProperty("AppPropertiesDir") + File.separator +
	// PROPERTIES_RESOURCE_LOCATION));
	// Properties p = propertiesFactoryBean.getObject();
	// String[] activeProfiles = env.getActiveProfiles();
	// p.setProperty("profile", activeProfiles[0]);
	// return p;
	// }

	@Bean
	public PasswordEncoder passwordEncoder() {
		return new BCryptPasswordEncoder();
	}

}
