package gov.va.vss.config;

import javax.sql.DataSource;

import org.springframework.batch.core.configuration.annotation.DefaultBatchConfigurer;
import org.springframework.batch.core.configuration.annotation.EnableBatchProcessing;
import org.springframework.batch.core.configuration.support.ApplicationContextFactory;
import org.springframework.batch.core.configuration.support.GenericApplicationContextFactory;
import org.springframework.batch.core.repository.JobRepository;
import org.springframework.batch.core.repository.support.JobRepositoryFactoryBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.transaction.PlatformTransactionManager;

import gov.va.vss.config.batchJobs.JobConfigurationTicker;

@Configuration
@EnableBatchProcessing(modular = true)
public class BatchConfig extends DefaultBatchConfigurer {

	@Autowired
	private DataSource dataSource;
	@Autowired
	private PlatformTransactionManager transactionManager;

	@Override
	public PlatformTransactionManager getTransactionManager() {
		return transactionManager;
	}

	@Override
	protected JobRepository createJobRepository() throws Exception {
		JobRepositoryFactoryBean f = new JobRepositoryFactoryBean();
		f.setDataSource(dataSource);
		f.setDatabaseType("SQLSERVER");
		f.setTransactionManager(transactionManager);
		f.setTablePrefix("CORE.BATCH_");
		// f.setIsolationLevelForCreate("ISOLATION_DEFAULT");
		f.afterPropertiesSet();
		return f.getObject();
	}

	/* Sample job */
	@Bean
	public ApplicationContextFactory tickerSampleJob() {
		return new GenericApplicationContextFactory(JobConfigurationTicker.class);
	}

}
