package gov.va.vss.model;

import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.NONE;
import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.PUBLIC_ONLY;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.builder.CompareToBuilder;
import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;
import org.hibernate.annotations.BatchSize;
import org.hibernate.validator.constraints.Length;
import org.hibernate.validator.constraints.NotBlank;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonView;

import gov.va.shared.model.core.AbstractAuditedVersionedPersistent;
import gov.va.vss.model.facility.Facility;

@Entity
@Table(name = "VssBenefitingServiceRoles", schema = "dbo")
@JsonAutoDetect(fieldVisibility = NONE, getterVisibility = PUBLIC_ONLY, isGetterVisibility = PUBLIC_ONLY)
public class BenefitingServiceRole extends AbstractAuditedVersionedPersistent<BenefitingServiceRole>
		implements Comparable<BenefitingServiceRole> {
	private static final long serialVersionUID = 6904844123870655771L;

	/**
	 * See @JsonView documentation - CPB
	 */
	public static class BenefitingServiceRoleView {
		public interface Basic {
		}

		public interface Extended extends Basic {
		}
	}

	// -------------------------------------- Fields

	@NotBlank
	@Length(max = 50)
	private String name;
	private String location;

	private Facility facility;
	
	private BenefitingService benefitingService;

	private List<FacilityAssignment> facilityAssignments;

	// -------------------------------------- Business Methods

	@Transient
	@JsonView(BenefitingServiceRoleView.Basic.class)
	public String getDisplayName() {
		return getName() + (StringUtils.isBlank(location) ? "" : " - " + location);
	}

	// -------------------------------------- Common Methods

	@Override
	protected boolean requiredEquals(BenefitingServiceRole oo) {
		return new EqualsBuilder().append(getName(), oo.getName()).append(getLocation(), oo.getLocation())
				.append(nullSafeGetId(getBenefitingService()), nullSafeGetId(oo.getBenefitingService())).isEquals();
	}

	@Override
	protected int requiredHashCode() {
		return new HashCodeBuilder().append(getName()).append(getLocation())
				.append(nullSafeGetId(getBenefitingService())).toHashCode();
	}

	@Override
	public int compareTo(BenefitingServiceRole o) {
		if (equals(o))
			return 0;

		return new CompareToBuilder().append(nullSafeLowercase(getDisplayName()), nullSafeLowercase(o.getDisplayName()))
				.toComparison() > 0 ? 1 : -1;
	}

	@Override
	public String toString() {
		return getDisplayName();
	}

	// -------------------------------------- Accessor Methods

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "FacilityFK", nullable = false)
	@JsonIgnore
	public Facility getFacility() {
		return facility;
	}

	public void setFacility(Facility facility) {
		this.facility = facility;
	}
	
	@Column(length = 50)
	@JsonView(BenefitingServiceRoleView.Extended.class)
	@Transient
	public String getLocation() {
		return location;
	}

	public void setLocation(String location) {
		this.location = location;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "VssBenefitingServicesFK", nullable = false)
	@JsonIgnore
	public BenefitingService getBenefitingService() {
		return benefitingService;
	}

	public void setBenefitingService(BenefitingService benefitingService) {
		this.benefitingService = benefitingService;
	}

	@Column(length = 50, nullable = false)
	@JsonView(BenefitingServiceRoleView.Extended.class)
	@Transient
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	@OneToMany(mappedBy = "benefitingServiceRole", fetch = FetchType.LAZY)
	@BatchSize(size = 500)
	@JsonIgnore
	public List<FacilityAssignment> getFacilityAssignments() {
		if (facilityAssignments == null)
			facilityAssignments = new ArrayList<>();
		return facilityAssignments;
	}

	public void setFacilityAssignments(List<FacilityAssignment> facilityAssignments) {
		this.facilityAssignments = facilityAssignments;
	}

}
