package gov.va.vss.model.jobExecution;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;

import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;

import gov.va.shared.model.core.AbstractAuditedVersionedPersistent;
import gov.va.vss.model.lookup.JobExecutionType;

/**
 * Class representing parameters which may be of interest to batch / scheduled
 * jobs
 */
@Entity
@Table(name = "JOB_EXECUTION_METADATA", schema = "CORE")
public class JobExecutionMetadata extends AbstractAuditedVersionedPersistent<JobExecutionMetadata> {
	private static final long serialVersionUID = 4416728108665979313L;

	// --------------------------------------- Fields

	@NotNull
	private JobExecutionType type;

	@NotNull
	private String name;

	private String value;

	// --------------------------------------- Business Methods

	// --------------------------------------- Common Methods

	@Override
	protected boolean requiredEquals(JobExecutionMetadata u) {
		return new EqualsBuilder().append(getName(), u.getName())
				.append(nullSafeGetId(getType()), nullSafeGetId(u.getType())).isEquals();
	}

	@Override
	protected int requiredHashCode() {
		return new HashCodeBuilder().append(getName()).append(nullSafeGetId(getType())).toHashCode();
	}

	// --------------------------------------- Accessor Methods

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "JOB_EXECUTION_TYPE_ID")
	public JobExecutionType getType() {
		return type;
	}

	public void setType(JobExecutionType type) {
		this.type = type;
	}

	@Column(name = "JOB_NAME", nullable = false)
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	@Column(name = "JOB_VALUE")
	public String getValue() {
		return value;
	}

	public void setValue(String value) {
		this.value = value;
	}

}
