package gov.va.vss.persistence.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.Query;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Repository;

import gov.va.shared.model.lookup.sds.State;
import gov.va.vss.model.Donor;
import gov.va.vss.model.lookup.VtkDonorType;
import gov.va.vss.model.lookup.VtkDonorType.VtkDonorTypeValue;
import gov.va.vss.persistence.dao.DonorDAO;
import gov.va.vss.persistence.queryCustomization.QueryCustomization;

@Repository
public class DonorDAOImpl extends GenericHibernateSortedDAOImpl<Donor> implements DonorDAO {
	@SuppressWarnings("unused")
	private static final Logger log = LoggerFactory.getLogger(DonorDAOImpl.class);

	@Value("${donorSearch.maxResults}")
	private int maxResults;
	
	@SuppressWarnings("unchecked")
	@Override
	public List<Donor> findByCriteria(VtkDonorType donorType, String firstName, String middleName, String lastName, 
			String city, State state, String zip, String email, String phone, QueryCustomization... customization) {		
		
		StringBuilder sb = new StringBuilder("select d from ").append(Donor.class.getName()).append(" d");

		/* Don't bother with this yet - CPB */
		QueryCustomization cust = new QueryCustomization();

		List<String> whereClauseItems = new ArrayList<>();
		Map<String, Object> params = new HashMap<>();

		if (donorType == null) {
			throw new IllegalArgumentException("donorType cannot be null");
		}
		
		whereClauseItems.add("d.vtkDonorType.id = :donorTypeId");
		params.put("donorTypeId", donorType.getId());
		
		// Continue searching in donor table ONLY if donor type is individual
		if (donorType.getLookupType() != VtkDonorTypeValue.INDIVIDUAL) {
			return new ArrayList<Donor>();
		}
		
		if (StringUtils.isNotBlank(firstName)) {
			whereClauseItems.add("LOWER(d.firstName) like :firstName");
			params.put("firstName", "%" + firstName.toLowerCase() + "%");
		}
		
		if (StringUtils.isNotBlank(middleName)) {
			whereClauseItems.add("LOWER(d.middleName) like :middleName");
			params.put("middleName", "%" + middleName.toLowerCase() + "%");
		}

		if (StringUtils.isNotBlank(lastName)) {
			whereClauseItems.add("LOWER(d.lastName) like :lastName");
			params.put("lastName", "%" + lastName.toLowerCase() + "%");
		}

		if (StringUtils.isNotBlank(email)) {
			whereClauseItems.add("LOWER(d.email) like :email");
			params.put("email", "%" + email.toLowerCase() + "%");
		}
		
		if (StringUtils.isNotBlank(city)) {
			whereClauseItems.add("LOWER(d.city) like :city");
			params.put("city", "%" + city.toLowerCase() + "%");
		}

		if (state != null) {
			whereClauseItems.add("d.state.id = :stateId");
			params.put("stateId", state.getId());
		}
		
		if (StringUtils.isNotBlank(zip)) {
			whereClauseItems.add("LOWER(d.zip) like :zip");
			params.put("zip", "%" + zip.toLowerCase() + "%");
		}

		if (StringUtils.isNotBlank(phone)) {
			whereClauseItems.add("LOWER(d.phone) like :phone");
			params.put("phone", "%" + phone.toLowerCase() + "%");
		}

		Query q = constructQuery(em, sb, whereClauseItems, params, null, cust);

		int maxResults = this.maxResults;
		if (cust.getRowLimitation() != null)
			maxResults = cust.getRowLimitation();

		List<Donor> resultList = q.setMaxResults(maxResults).getResultList();
		return resultList;
	}

}
