package gov.va.vss.service.impl;

import org.springframework.batch.core.explore.JobExplorer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.MessageSource;
import org.springframework.transaction.annotation.Transactional;

import gov.va.shared.model.lookup.sds.VAFacility;
import gov.va.shared.persistence.AppUserDAO;
import gov.va.shared.persistence.lookup.sds.GenderDAO;
import gov.va.shared.persistence.lookup.sds.StateDAO;
import gov.va.shared.persistence.lookup.sds.VAFacilityDAO;
import gov.va.shared.persistence.lookup.sds.VAFacilityTypeDAO;
import gov.va.shared.util.SessionUtil;
import gov.va.vss.model.facility.Facility;
import gov.va.vss.persistence.dao.ApplicationParametersDAO;
import gov.va.vss.persistence.dao.BinaryObjectDAO;
import gov.va.vss.persistence.dao.DonationDetailDAO;
import gov.va.vss.persistence.dao.DonationSummaryDAO;
import gov.va.vss.persistence.dao.DonorDAO;
import gov.va.vss.persistence.dao.ExcludedEntityDAO;
import gov.va.vss.persistence.dao.FacilityAssignmentDAO;
import gov.va.vss.persistence.dao.FacilityDAO;
import gov.va.vss.persistence.dao.LetterTemplateDAO;
import gov.va.vss.persistence.dao.LocationDAO;
import gov.va.vss.persistence.dao.NationalOfficialDAO;
import gov.va.vss.persistence.dao.OrganizationDAO;
import gov.va.vss.persistence.dao.ParkingStickerDAO;
import gov.va.vss.persistence.dao.ServiceParametersDAO;
import gov.va.vss.persistence.dao.TemplateDAO;
import gov.va.vss.persistence.dao.UniformDAO;
import gov.va.vss.persistence.dao.VistaStationDataDAO;
import gov.va.vss.persistence.dao.VoluntaryServiceStaffDAO;
import gov.va.vss.persistence.dao.VolunteerDAO;
import gov.va.vss.persistence.dao.VolunteerFacilityAssignmentDAO;
import gov.va.vss.persistence.dao.VolunteerOrganizationDAO;
import gov.va.vss.persistence.dao.VtkUserDAO;
import gov.va.vss.persistence.dao.WorkEntryDAO;
import gov.va.vss.persistence.dao.audit.AuditLogEntryDAO;
import gov.va.vss.persistence.dao.jobExecution.JobExecutionMetadataDAO;
import gov.va.vss.persistence.dao.jobExecution.JobExecutionResultDAO;
import gov.va.vss.persistence.dao.jobExecution.JobExecutionResultItemDAO;
import gov.va.vss.persistence.dao.lookup.ActiveStatusDAO;
import gov.va.vss.persistence.dao.lookup.FacilityTypeDAO;
import gov.va.vss.persistence.dao.lookup.VtkAdministrativeUnitDAO;
import gov.va.vss.persistence.dao.views.FacilityAndVisnDAO;
import gov.va.vss.service.email.EmailService;

@Transactional
public class AbstractServiceImpl {

	@Autowired
	protected ActiveStatusDAO activeStatusDAO;
	@Autowired
	protected ApplicationParametersDAO applicationParametersDAO;
	@Autowired
	protected AppUserDAO appUserDAO;
	@Autowired
	protected AuditLogEntryDAO auditLogEntryDAO;
	@Autowired
	protected BinaryObjectDAO binaryObjectDAO;
	@Autowired
	protected DonationDetailDAO donationDetailDAO;
	@Autowired
	protected DonationSummaryDAO donationSummaryDAO;
	@Autowired
	protected DonorDAO donorDAO;
	@Autowired
	protected ExcludedEntityDAO excludedEntityDAO;
	@Autowired
	protected GenderDAO genderDAO;
	@Autowired
	protected FacilityAndVisnDAO facilityAndVisnDAO;
	@Autowired
	protected FacilityAssignmentDAO facilityAssignmentDAO;
	@Autowired
	protected JobExecutionMetadataDAO jobExecutionMetadataDAO;
	@Autowired
	protected JobExecutionResultDAO jobExecutionResultDAO;
	@Autowired
	protected JobExecutionResultItemDAO jobExecutionResultItemDAO;
	@Autowired
	protected LetterTemplateDAO letterTemplateDAO;
	@Autowired
	protected LocationDAO locationDAO;
	@Autowired
	protected NationalOfficialDAO nationalOfficialDAO;
	@Autowired
	protected OrganizationDAO organizationDAO;
	@Autowired
	protected ParkingStickerDAO parkingStickerDAO;
	@Autowired
	protected StateDAO stateDAO;
	@Autowired
	protected TemplateDAO templateDAO;
	@Autowired
	protected UniformDAO uniformDAO;
	@Autowired
	protected VAFacilityDAO vaFacilityDAO;
	@Autowired
	protected VAFacilityTypeDAO vaFacilityTypeDAO;
	@Autowired
	protected VistaStationDataDAO vistaStationDataDAO;
	@Autowired
	protected VolunteerDAO volunteerDAO;
	@Autowired
	protected VolunteerFacilityAssignmentDAO volunteerFacilityAssignmentDAO;
	@Autowired
	protected VolunteerOrganizationDAO volunteerOrganizationDAO;
	@Autowired
	protected VtkAdministrativeUnitDAO vtkAdministrativeUnitDAO;
	@Autowired
	protected FacilityDAO facilityDAO;
	@Autowired
	protected FacilityTypeDAO facilityTypeDAO;
	@Autowired
	protected ServiceParametersDAO serviceParametersDAO;
	@Autowired
	protected VoluntaryServiceStaffDAO voluntaryServiceStaffDAO;
	@Autowired
	protected VtkUserDAO vtkUserDAO;
	@Autowired
	protected WorkEntryDAO workEntryDAO;
	
	@Autowired
	protected EmailService emailService;
	@Autowired
	protected MessageSource messageSource;

	@Autowired
	protected JobExplorer jobExplorer;

	protected VAFacility getSiteContext() {
		return SessionUtil.getSiteContext();
	}

	protected Facility getFacilityContext() {
		VAFacility siteContext = getSiteContext();
		if (siteContext == null)
			return null;
		return facilityDAO.findByVAFacility(siteContext.getId());
	}
	
}
