package gov.va.vss.web;

import static gov.va.shared.util.StationsUtil.isThreeDigitStation;

import java.util.Locale;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import javax.persistence.OptimisticLockException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.batch.core.explore.JobExplorer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.MessageSource;
import org.springframework.core.env.Environment;
import org.springframework.http.HttpStatus;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.support.RequestContextUtils;

import gov.va.shared.model.AppUser;
import gov.va.shared.model.Permission.PermissionType;
import gov.va.shared.model.lookup.sds.VAFacility;
import gov.va.shared.persistence.AppUserDAO;
import gov.va.shared.persistence.AppUserFacilityDAO;
import gov.va.shared.persistence.lookup.sds.GenderDAO;
import gov.va.shared.persistence.lookup.sds.StateDAO;
import gov.va.shared.persistence.lookup.sds.VAFacilityDAO;
import gov.va.shared.service.AppUserService;
import gov.va.shared.service.VAFacilityService;
import gov.va.shared.util.SecurityUtil;
import gov.va.shared.util.ServletUtil;
import gov.va.shared.util.SessionUtil;
import gov.va.shared.util.StationsUtil;
import gov.va.vss.model.ObjectScopedToStationNumbers;
import gov.va.vss.model.facility.Facility;
import gov.va.vss.persistence.dao.AwardDAO;
import gov.va.vss.persistence.dao.BenefitingServiceDAO;
import gov.va.vss.persistence.dao.BenefitingServiceRoleDAO;
import gov.va.vss.persistence.dao.DonationDetailDAO;
import gov.va.vss.persistence.dao.DonationReferenceDAO;
import gov.va.vss.persistence.dao.DonationSummaryDAO;
import gov.va.vss.persistence.dao.DonorDAO;
import gov.va.vss.persistence.dao.FacilityAssignmentDAO;
import gov.va.vss.persistence.dao.FacilityDAO;
import gov.va.vss.persistence.dao.LetterTemplateDAO;
import gov.va.vss.persistence.dao.LocationDAO;
import gov.va.vss.persistence.dao.NationalOfficialDAO;
import gov.va.vss.persistence.dao.OrganizationDAO;
import gov.va.vss.persistence.dao.ParkingStickerDAO;
import gov.va.vss.persistence.dao.UniformDAO;
import gov.va.vss.persistence.dao.VistaStationDataDAO;
import gov.va.vss.persistence.dao.VoluntaryServiceStaffDAO;
import gov.va.vss.persistence.dao.VolunteerDAO;
import gov.va.vss.persistence.dao.VolunteerFacilityAssignmentDAO;
import gov.va.vss.persistence.dao.VolunteerOrganizationDAO;
import gov.va.vss.persistence.dao.WorkEntryDAO;
import gov.va.vss.persistence.dao.jobExecution.JobExecutionResultDAO;
import gov.va.vss.persistence.dao.lookup.ActiveStatusDAO;
import gov.va.vss.persistence.dao.lookup.FacilityTypeDAO;
import gov.va.vss.persistence.dao.lookup.JobExecutionTypeDAO;
import gov.va.vss.persistence.dao.lookup.NACStatusDAO;
import gov.va.vss.persistence.dao.lookup.OrganizationTypeDAO;
import gov.va.vss.persistence.dao.lookup.StdCreditCardTypeDAO;
import gov.va.vss.persistence.dao.lookup.StdVAVSTitleDAO;
import gov.va.vss.persistence.dao.lookup.TransportationMethodDAO;
import gov.va.vss.persistence.dao.lookup.VtkAdministrativeUnitDAO;
import gov.va.vss.persistence.dao.lookup.VtkDonationTypeDAO;
import gov.va.vss.persistence.dao.lookup.VtkLanguageDAO;
import gov.va.vss.persistence.dao.lookup.VtkStaffTitleDAO;
import gov.va.vss.persistence.dao.lookup.VtkVoluntaryServiceTypeDAO;
import gov.va.vss.persistence.dao.views.CombinedInstitutionDAO;
import gov.va.vss.service.DonationService;
import gov.va.vss.service.DonorService;
import gov.va.vss.service.FacilityService;
import gov.va.vss.service.LetterTemplateService;
import gov.va.vss.service.LocationService;
import gov.va.vss.service.NationalOfficialService;
import gov.va.vss.service.OrganizationService;
import gov.va.vss.service.ParkingStickerService;
import gov.va.vss.service.ServiceParametersService;
import gov.va.vss.service.UniformService;
import gov.va.vss.service.VoluntaryServiceStaffService;
import gov.va.vss.service.VolunteerService;
import gov.va.vss.service.batch.BatchProcessService;
import gov.va.vss.util.DateUtil;
import gov.va.vss.web.ajax.AjaxRequestHandler;
import gov.va.vss.web.validation.WebValidationService;

public abstract class AbstractAppController {
	private static final Logger log = LoggerFactory.getLogger(AbstractAppController.class);

	// ----------------------------------------- Static Fields and Constants

	public static final String DEFAULT_COMMAND_NAME = "command";

	public static final String FORM_READ_ONLY = "FORM_READ_ONLY";

	// ----------------------------------------- Fields

	// -------------------------- DAOs

	@Autowired
	protected ActiveStatusDAO activeStatusDAO;
	@Autowired
	protected AppUserFacilityDAO appUserFacilityDAO;
	@Autowired
	protected AppUserDAO appUserDAO;
	@Autowired
	protected AwardDAO awardDAO;
	@Autowired
	protected BenefitingServiceDAO benefitingServiceDAO;
	@Autowired
	protected BenefitingServiceRoleDAO benefitingServiceRoleDAO;
	@Autowired
	protected CombinedInstitutionDAO combinedInstitutionDAO;
	@Autowired
	protected DonationDetailDAO donationDetailDAO;
	@Autowired
	protected DonationSummaryDAO donationSummaryDAO;
	@Autowired
	protected DonationReferenceDAO donationReferenceDAO;
	@Autowired
	protected DonorDAO donorDAO;
	@Autowired
	protected FacilityDAO facilityDAO;
	@Autowired
	protected FacilityAssignmentDAO facilityAssignmentDAO;
	@Autowired
	protected FacilityTypeDAO facilityTypeDAO;
	@Autowired
	protected GenderDAO genderDAO;
	@Autowired
	protected JobExecutionTypeDAO jobExecutionTypeDAO;
	@Autowired
	protected JobExecutionResultDAO jobExecutionResultDAO;
	@Autowired
	protected LetterTemplateDAO letterTemplateDAO;
	@Autowired
	protected LocationDAO locationDAO;
	@Autowired
	protected NACStatusDAO nacStatusDAO;
	@Autowired
	protected NationalOfficialDAO nationalOfficialDAO;
	@Autowired
	protected OrganizationDAO organizationDAO;
	@Autowired
	protected OrganizationTypeDAO vtkOrganizationTypeDAO;
	@Autowired
	protected ParkingStickerDAO parkingStickerDAO;
	@Autowired
	protected StateDAO stateDAO;
	@Autowired
	protected StdCreditCardTypeDAO stdCreditCardTypeDAO;
	@Autowired
	protected StdVAVSTitleDAO stdVAVSTitleDAO;
	@Autowired
	protected TransportationMethodDAO transportationMethodDAO;
	@Autowired
	protected UniformDAO uniformDAO;
	@Autowired
	protected VAFacilityDAO vaFacilityDAO;
	@Autowired
	protected VistaStationDataDAO vistaStationDataDAO;
	@Autowired
	protected VoluntaryServiceStaffDAO voluntaryServiceStaffDAO;
	@Autowired
	protected VolunteerDAO volunteerDAO;
	@Autowired
	protected VolunteerFacilityAssignmentDAO volunteerFacilityAssignmentDAO;
	@Autowired
	protected VolunteerOrganizationDAO volunteerOrganizationDAO;
	@Autowired
	protected VtkAdministrativeUnitDAO vtkAdministrativeUnitDAO;
	@Autowired
	protected VtkDonationTypeDAO vtkDonationTypeDAO;
	@Autowired
	protected VtkLanguageDAO vtkLanguageDAO;
	@Autowired
	protected VtkVoluntaryServiceTypeDAO vtkVoluntaryServiceTypeDAO;
	@Autowired
	protected VtkStaffTitleDAO vtkStaffTitleDAO;
	@Autowired
	protected WorkEntryDAO workEntryDAO;

	// -------------------------- Services

	@Autowired
	protected AppUserService appUserService;
	@Autowired
	protected BatchProcessService batchProcessServiceImpl;
	@Autowired
	protected DonationService donationService;
	@Autowired
	protected DonorService donorService;
	@Autowired
	protected FacilityService facilityService;
	@Autowired
	protected LetterTemplateService letterTemplateService;
	@Autowired
	protected LocationService locationService;
	@Autowired
	protected NationalOfficialService nationalOfficialService;
	@Autowired
	protected OrganizationService organizationService;
	@Autowired
	protected ParkingStickerService parkingStickerService;
	@Autowired
	protected ServiceParametersService voluntaryServiceParametersService;
	@Autowired
	protected UniformService uniformService;
	@Autowired
	protected VAFacilityService vaFacilityService;
	@Autowired
	protected VoluntaryServiceStaffService voluntaryServiceStaffService;
	@Autowired
	protected VolunteerService volunteerService;

	// -------------------------- Others

	@Autowired
	protected AjaxRequestHandler ajaxRequestHandler;
	@Autowired
	protected DateUtil dateUtil;
	@Autowired
	protected Environment env;
	@Autowired
	protected MessageSource messageSource;
	@Autowired
	protected JobExplorer jobExplorer;
	@Autowired
	protected UserNotifier userNotifier;
	@Autowired
	protected WebValidationService webValidationService;

	protected boolean isAjax(HttpServletRequest r) {
		return AjaxRequestHandler.isAjax(r);
	}

	protected String getMessage(String code) {
		return getMessage(code, new Object[] {});
	}

	protected String getMessage(String code, Object[] args) {
		return messageSource.getMessage(code, args, Locale.getDefault());
	}

	@ExceptionHandler(AccessDeniedException.class)
	public ModelAndView processAuthorizationException(AccessDeniedException ex) {
		return new ModelAndView("authorizationException", "exceptionStackTrace", ExceptionUtils.getFullStackTrace(ex));
	}

	@ExceptionHandler(OptimisticLockException.class)
	public ModelAndView processOptimisticLockException(OptimisticLockException ex) {
		return new ModelAndView("optimisticLockException", "exceptionStackTrace", ExceptionUtils.getFullStackTrace(ex));
	}

	@ExceptionHandler(Throwable.class)
	public ModelAndView processError(Throwable ex, HttpServletRequest request, HttpServletResponse response) {
		if (isAjax(request)) {
			/*
			 * Necessary to trigger the jQuery error() handler as opposed to the
			 * success() handler - CPB
			 */
			response.setStatus(HttpStatus.BAD_REQUEST.value());
			return ajaxRequestHandler.getExceptionModelAndView(ex, request);
		} else {
			return new ModelAndView("error", "exceptionStackTrace", ExceptionUtils.getFullStackTrace(ex));
		}
	}

	// ----------------------------------------- Utility Methods

	protected void requirePermissionsAtFacility(long vaFacilityId, PermissionType... requiredPermissionsAtStation) {
		if (ArrayUtils.isNotEmpty(requiredPermissionsAtStation)) {
			SecurityUtil.ensureAllPermissionsAtFacility(vaFacilityId, requiredPermissionsAtStation);
		}
	}

	protected void requirePermissionsAtCurrentFacility(PermissionType... requiredPermissionsAtFacility) {
		if (ArrayUtils.isNotEmpty(requiredPermissionsAtFacility)) {
			SecurityUtil.ensureAllPermissionsAtCurrentFacility(requiredPermissionsAtFacility);
		}
	}

	protected void setFormAsReadOnly(ModelMap model, boolean readOnly) {
		model.put(FORM_READ_ONLY, readOnly);
	}

	protected void updateModelForPermissionsIfNecessary(ModelMap model, PermissionType... permissions) {
		boolean userHasPermissions = SecurityUtil.hasAllPermissionsAtCurrentFacility(permissions);
		setFormAsReadOnly(model, !userHasPermissions);
	}

	/**
	 * Changes the user's station to the first approved item (following natural
	 * iteration order) in the specified requestedInstitutions if their session
	 * isn't already set to one of those stations.
	 * 
	 * National Administrators are considered implicitly to have all stations
	 * assigned; if the current user is a National Administrator this method is
	 * a no-op and null is returned.
	 * 
	 * @param requestedInstitutions
	 * @param request
	 * @param requestedInstitutionsHaveThreeDigitScope
	 *            If true, then the requestedInstitutions specified may ONLY be
	 *            those with three-digit station numbers. The system will select
	 *            the first station assigned to the user whose first three
	 *            digits match any of the requestedInstitutions (even if the
	 *            user's assigned station has more than three digits).
	 * @param requiredPermissionsAtStation
	 *            If non-null, the user must hold all the specified permissions
	 *            at the VAFacility (either via global role assignments or
	 *            facility-specific overrides) to qualify that station for
	 *            selection during the update process.
	 * @return The newly changed station, or null if the currently selected
	 *         station already belongs to the requested institutions per the
	 *         logic above.
	 * @throws SecurityException
	 *             If the user's current station is not part of the
	 *             requestedInstitutions collection and the user has no
	 *             qualifying assigned station to which their session can be
	 *             changed.
	 * @throws IllegalArgumentException
	 *             If the requestedInstitutionsHaveThreeDigitScope is true but
	 *             one of the specified requestedInstitutions has a station
	 *             number greater than three digits long.
	 */
	protected VAFacility updateStationIfNecessary(ObjectScopedToStationNumbers obj, HttpServletRequest request,
			boolean requestedInstitutionsHaveThreeDigitScope, PermissionType... requiredPermissionsAtStation)
			throws SecurityException, IllegalArgumentException {
		AppUser user = SecurityUtil.getCurrentUser();
		boolean isNationalAdmin = user.isNationalAdmin();

		SortedSet<String> requestedStationNumbers = new TreeSet<>();
		for (String stationNumber : obj.getScopedToStationNumbers()) {
			if (!isNationalAdmin && requestedInstitutionsHaveThreeDigitScope
					&& !StationsUtil.isThreeDigitStation(stationNumber))
				throw new IllegalArgumentException("An institution with station number '" + stationNumber
						+ "' was specified but only three-digit stations may be specified"
						+ " when the requestedInstitutionsHaveThreeDigitScope is set to true.");
			requestedStationNumbers.add(stationNumber);
		}

		VAFacility currentVAFacility = getSiteContext();
		if (requestedStationNumbers.contains(requestedInstitutionsHaveThreeDigitScope ? currentVAFacility.getSta3n()
				: currentVAFacility.getStationNumber()))
			return null;

		VAFacility matchingFacility = null;

		if (isNationalAdmin) {
			String requestedStation = requestedStationNumbers.iterator().next();
			matchingFacility = vaFacilityDAO.findByStationNumber(requestedStation);
		} else {
			Set<VAFacility> userFacilities = user.getBasicFacilities();

			outer: //
			for (String requestedStationNumber : requestedStationNumbers) {
				for (VAFacility fa : userFacilities) {

					if (requiredPermissionsAtStation != null
							&& !SecurityUtil.hasAllPermissionsAtFacility(fa.getId(), requiredPermissionsAtStation))
						continue;

					if (requestedInstitutionsHaveThreeDigitScope) {
						/*
						 * Any institution assigned to the user whose first
						 * three digits match one of the requestedStationNumbers
						 * qualifies. Give preference to an exact
						 * three-digit-to-three-digit match. CPB
						 */
						boolean matches = fa.getSta3n().equals(requestedStationNumber);
						if (matches) {
							if (matchingFacility == null || (!isThreeDigitStation(matchingFacility.getStationNumber())
									&& isThreeDigitStation(fa.getStationNumber())))
								matchingFacility = fa;
						}
					} else {
						/*
						 * The first institutions assigned to the user whose
						 * complete station number exactly matches one of the
						 * requestedStationNumbers qualifies - CPB
						 */
						boolean matches = fa.getStationNumber().equals(requestedStationNumber);
						if (matches) {
							matchingFacility = fa;
							break outer;
						}
					}
				}
			}
		}

		if (matchingFacility != null) {
			SessionUtil.setSiteContext(matchingFacility);
			userNotifier.notifyUserOnce(request, "generic.stationChanged");
			return matchingFacility;
		}

		throw new SecurityException("The user does not have access to any of the following station(s): {"
				+ StringUtils.join(requestedStationNumbers, ", ") + "}");
	}

	protected Locale getLocale(HttpServletRequest portletRequest) {
		HttpServletRequest request = ServletUtil.getThreadBoundServletRequest();
		Locale locale = request == null ? Locale.getDefault() : RequestContextUtils.getLocale(request);
		return locale;
	}

	protected AppUser getCurrentUser() {
		return SecurityUtil.getCurrentUser();
	}

	protected String getCurrentUserName() {
		return SecurityUtil.getCurrentUserName();
	}

	protected VAFacility getSiteContext() {
		return SessionUtil.getSiteContext();
	}

	protected Facility getFacilityContext() {
		VAFacility siteContext = getSiteContext();
		if (siteContext == null)
			return null;
		Facility f = facilityDAO.findByVAFacility(siteContext.getId());
		if (f == null)
			throw new IllegalStateException(
					"No Facility is currently linked to the VA site " + siteContext.getStationNumber()
							+ "; please contact support to remedy this via the Facility Management screen.");
		return f;
	}

}
