package gov.va.vss.web.donation;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.TreeSet;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.bind.support.SessionStatus;

import gov.va.shared.model.Permission;
import gov.va.shared.util.WebUtil;
import gov.va.vss.model.DonGenPostFund;
import gov.va.vss.model.DonationDetail;
import gov.va.vss.model.DonationReference;
import gov.va.vss.model.DonationSummary;
import gov.va.vss.model.Donor;
import gov.va.vss.model.facility.Facility;
import gov.va.vss.service.validation.ServiceValidationException;
import gov.va.vss.web.AbstractAppController;
import gov.va.vss.web.ValidationException;
import gov.va.vss.web.breadcrumbs.Breadcrumb;

@Controller
@SessionAttributes(value = { AbstractAppController.DEFAULT_COMMAND_NAME })
public class DonationController extends AbstractAppController {
	@Autowired
	private DonationValidator donationValidator;

	@RequestMapping("/donationEdit.htm")
	@Breadcrumb("Edit Donation")
	@PreAuthorize("hasAuthority('" + Permission.DONATION_READ + "')")
	public String editDonation(@RequestParam long id, ModelMap model, HttpServletRequest request) {
		DonationSummary donation = donationSummaryDAO.findRequiredByPrimaryKey(id);

		DonationCommand command = new DonationCommand(donation);
		command.setCurrentFiscalYearStartDate(dateUtil.getCurrentFiscalYearStartDate());
		command.setCurrentFiscalYearEndDate(dateUtil.getCurrentFiscalYearEndDate());

		model.addAttribute(DEFAULT_COMMAND_NAME, command);
		createReferenceData(model, command, false);
		// model.addAttribute("newDonationSummary", false);
		return "editDonation";
	}

	@RequestMapping("/donationCreate.htm")
	@Breadcrumb("Create Donation")
	@PreAuthorize("hasAuthority('" + Permission.DONATION_CREATE + "')")
	public String createDonation(@RequestParam long donorId, ModelMap model, HttpServletRequest request) {
		DonationSummary donation = new DonationSummary();
		Facility facility = getFacilityContext();
		donation.setFacility(facility);

		Donor donor = donorDAO.findRequiredByPrimaryKey(donorId);
		donation.setDonor(donor);

		DonationReference donReference = donationReferenceDAO
				.findByInstitutionAndRef(getSiteContext().getStationNumber(), "none");

		if (donation.getDonReference() == null)
			donation.setDonReference(donReference);

		DonationCommand command = new DonationCommand(donation);
		command.setCurrentFiscalYearStartDate(dateUtil.getCurrentFiscalYearStartDate());
		command.setCurrentFiscalYearEndDate(dateUtil.getCurrentFiscalYearEndDate());

		model.addAttribute(DEFAULT_COMMAND_NAME, command);
		createReferenceData(model, command, false);
		return "createDonation";
	}

	@RequestMapping("/donationDetails")
	@PreAuthorize("hasAuthority('" + Permission.DONATION_READ + "')")
	public @ResponseBody List<DonationDetail> getDonationDetailsList(@RequestParam long donationSummaryId) {
		if (donationSummaryId == 0)
			return new ArrayList<DonationDetail>();
		List<DonationDetail> donationDetailList = donationDetailDAO.findByCriteria(donationSummaryId);
		return donationDetailList;
	}

	@RequestMapping("/donationDelete.htm")
	@PreAuthorize("hasAuthority('" + Permission.DONATION_CREATE + "')")
	public String deleteDonationSummary(@RequestParam long donationSummaryId, @RequestParam long donorId)
			throws ServiceValidationException {
		donationService.deleteDonationSummary(donationSummaryId);

		return "redirect:/donorEdit.htm?id=" + donorId;
	}

	@RequestMapping("/deleteDonationDetail")
	@PreAuthorize("hasAuthority('" + Permission.DONATION_CREATE + "')")
	public @ResponseBody boolean deleteDonationDetail(@RequestParam long donationDetailId)
			throws ServiceValidationException {
		donationService.deleteDonationDetail(donationDetailId);
		return true;
	}

	@RequestMapping("/donationSummarySubmit.htm")
	@PreAuthorize("hasAuthority('" + Permission.DONATION_CREATE + "')")
	public String submitDonationSummary(@ModelAttribute(DEFAULT_COMMAND_NAME) DonationCommand command,
			BindingResult result, SessionStatus status, ModelMap model, HttpServletRequest request)
					throws ValidationException {
		DonationSummary donationSummary = command.getDonationSummary();

		List<DonationDetail> donationDetails = new ArrayList<DonationDetail>();
		if (command.getDonationDetail1() != null && command.getDonationDetail1().getDonationValue() != null
				&& command.getDonationDetail1().getDonationValue().doubleValue() > 0) {
			command.getDonationDetail1().setDonationSummary(donationSummary);
			donationDetails.add(command.getDonationDetail1());
		}
		if (command.getDonationDetail2() != null && command.getDonationDetail2().getDonationValue() != null
				&& command.getDonationDetail2().getDonationValue().doubleValue() > 0) {
			command.getDonationDetail2().setDonationSummary(donationSummary);
			donationDetails.add(command.getDonationDetail2());
		}
		if (command.getDonationDetail3() != null && command.getDonationDetail3().getDonationValue() != null
				&& command.getDonationDetail3().getDonationValue().doubleValue() > 0) {
			command.getDonationDetail3().setDonationSummary(donationSummary);
			donationDetails.add(command.getDonationDetail3());
		}

		if ((donationSummary.getVtkDonationType().getDonationType().equalsIgnoreCase("Item")
				|| donationSummary.getVtkDonationType().getDonationType().equalsIgnoreCase("Activity"))
				&& command.getDonationDetail4().getDonationValue() != null
				&& command.getDonationDetail4().getDonationValue().doubleValue() > 0) {
			command.getDonationDetail4().setDonationSummary(donationSummary);
			donationDetails.add(command.getDonationDetail4());
		}

		donationSummary.setDonationDetails(donationDetails);

		// Clean hidden fields
		donationSummary = cleanHiddenFields(donationSummary);

		/* Validation step (JSR303, other custom logic in the validator) */

		donationValidator.validate(command, result, false, "donationSummary");
		boolean hasErrors = result.hasErrors();

		if (!hasErrors) {
			try {
				donationSummary = donationService.saveOrUpdateDonationSummary(donationSummary);
				userNotifier.notifyUserOnceWithMessage(request, getMessage("donationSummary.update.success"));
			} catch (ServiceValidationException e) {
				webValidationService.handle(e, result);
				hasErrors = true;
			}
		}

		if (hasErrors) {
			createReferenceData(model, command, hasErrors);
			return "editDonation";
		} else {
			status.setComplete();
			return "redirect:/donationEdit.htm?id=" + donationSummary.getId();
		}
	}

	@RequestMapping("/donationDetailCreateOrUpdate")
	@PreAuthorize("hasAuthority('" + Permission.DONATION_CREATE + "')")
	public @ResponseBody DonationDetail createOrUpdateDonationDetail(@RequestParam Long donationSummaryId,
			@RequestParam(required = false) Long donationDetailId, @RequestParam DonGenPostFund generalPostFund,
			@RequestParam BigDecimal donationValue) throws ServiceValidationException {

		DonationDetail donationDetail = null;

		if (donationDetailId == null) {
			donationDetail = new DonationDetail();
			DonationSummary donationSummary = donationSummaryDAO.findRequiredByPrimaryKey(donationSummaryId);
			donationDetail.setDonationSummary(donationSummary);
		} else {
			donationDetail = donationDetailDAO.findRequiredByPrimaryKey(donationDetailId.longValue());
		}

		donationDetail.setDonGenPostFund(generalPostFund);
		donationDetail.setDonationValue(donationValue);
		donationDetail = donationService.saveOrUpdateDonationDetail(donationDetail);

		return donationDetail;
	}

	@RequestMapping("/changeDonor")
	@PreAuthorize("hasAuthority('" + Permission.DONATION_CREATE + "')")
	public @ResponseBody DonationSummary changeDonor(@RequestParam Long donationSummaryId, @RequestParam Long donorId)
			throws ServiceValidationException {

		DonationSummary donationSummary = donationSummaryDAO.findRequiredByPrimaryKey(donationSummaryId);
		Donor donor = donorDAO.findRequiredByPrimaryKey(donorId);
		donationSummary.setDonor(donor);
		donationSummary = donationService.saveOrUpdateDonationSummary(donationSummary);
		return donationSummary;
	}

	private void createReferenceData(ModelMap model, DonationCommand donationCommand, boolean hasErrors) {
		model.put("allDonationTypes", vtkDonationTypeDAO.findAllSorted());
		Facility facility = getFacilityContext();
		model.put("allDonationReferences", facility.getDonationReferences());
		model.put("allDonGenPostFunds", new TreeSet<DonGenPostFund>(facility.getDonGenPostFunds()));
		model.put("allCreditCardTypes", stdCreditCardTypeDAO.findAllSorted());
		WebUtil.addClassConstantsToModel(DonGenPostFund.class, model);

		if (!hasErrors && donationCommand.getDonationSummary() != null
				&& donationCommand.getDonationSummary().getDonationDate() != null) {
			boolean editable = dateUtil
					.isDonationSummaryEditable(donationCommand.getDonationSummary().getDonationDate());

			setFormAsReadOnly(model, !editable);
		}
	}

	private DonationSummary cleanHiddenFields(DonationSummary donationSummary) {
		// if the donation type = cash
		if (donationSummary.getVtkDonationType().getDonationType().equalsIgnoreCase("Cash")) {
			donationSummary.setCheckNumber(null);
			donationSummary.setCheckDate(null);
			donationSummary.setStdCreditCardType(null);
			donationSummary.setCreditCardTransactionId(null);
			donationSummary.setEpayTrackingID(null);
		}
		// if the donation type = Credit Card
		if (donationSummary.getVtkDonationType().getDonationType().equalsIgnoreCase("Credit Card")) {
			donationSummary.setCheckNumber(null);
			donationSummary.setCheckDate(null);
			donationSummary.setEpayTrackingID(null);
		}
		// if the donation type = Check
		if (donationSummary.getVtkDonationType().getDonationType().equalsIgnoreCase("Check")) {
			donationSummary.setStdCreditCardType(null);
			donationSummary.setCreditCardTransactionId(null);
			donationSummary.setEpayTrackingID(null);
		}
		// if the donation type = E-Donation
		if (donationSummary.getVtkDonationType().getDonationType().equalsIgnoreCase("E-Donation")) {
			donationSummary.setCheckNumber(null);
			donationSummary.setCheckDate(null);
			donationSummary.setStdCreditCardType(null);
			donationSummary.setCreditCardTransactionId(null);
		}
		// if the donation type = Item
		if (donationSummary.getVtkDonationType().getDonationType().equalsIgnoreCase("Item")) {
			donationSummary.setCheckNumber(null);
			donationSummary.setCheckDate(null);
			donationSummary.setStdCreditCardType(null);
			donationSummary.setCreditCardTransactionId(null);
			donationSummary.setEpayTrackingID(null);
			donationSummary.setInMemoryOf(null);
			donationSummary.setFamilyContact(null);
			donationSummary.setFamilyContactAddress(null);
			donationSummary.setFamilyContactCity(null);
			donationSummary.setFamilyContactState(null);
			donationSummary.setFamilyContactZip(null);
			donationSummary.setCc1(null);
			donationSummary.setCc2(null);
			donationSummary.setCc3(null);
			donationSummary.setCc4(null);
			donationSummary.setCc5(null);
		}
		// if the donation type = Activity
		if (donationSummary.getVtkDonationType().getDonationType().equalsIgnoreCase("Activity")) {
			donationSummary.setCheckNumber(null);
			donationSummary.setCheckDate(null);
			donationSummary.setStdCreditCardType(null);
			donationSummary.setCreditCardTransactionId(null);
			donationSummary.setEpayTrackingID(null);
			donationSummary.setInMemoryOf(null);
			donationSummary.setFamilyContact(null);
			donationSummary.setFamilyContactAddress(null);
			donationSummary.setFamilyContactCity(null);
			donationSummary.setFamilyContactState(null);
			donationSummary.setFamilyContactZip(null);
			donationSummary.setCc1(null);
			donationSummary.setCc2(null);
			donationSummary.setCc3(null);
			donationSummary.setCc4(null);
			donationSummary.setCc5(null);
		}
		return donationSummary;
	}

}
