package gov.va.vss.web.interceptor;

import java.util.List;
import java.util.Map;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.ModelAndView;

import gov.va.shared.util.SessionUtil;
import gov.va.vss.web.CommonController;
import gov.va.vss.web.breadcrumbs.Breadcrumbs;

@Component
public class BreadcrumbsInterceptor extends AbstractInterceptor {
	private static final Logger log = LoggerFactory.getLogger(BreadcrumbsInterceptor.class);

	// ------------------------------------- Static Fields

	private static final String REQUEST_BREADCRUMB_KEY = "request_key_breadcrumb";

	private static final String SESSION_ATTRIBUTE_BREADCRUMBS = "session_key_breadcrumbs";

	public static final String MODEL_ATTRIBUTE_NAME = "app_breadcrumbs";

	public static final String MOST_RECENT_BREADCRUMB_URL = "most_recent_breadcrumb";

	// ------------------------------------- Static Methods

	public static void setRequestBreadcrumb(ServletRequest request, String breadcrumb) {
		request.setAttribute(REQUEST_BREADCRUMB_KEY, breadcrumb);
	}

	// ------------------------------------- Business Methods

	@Override
	public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler,
			ModelAndView modelAndView) throws Exception {
		/*
		 * We don't want to add these attributes to specific ajax calls, but
		 * only to main page render calls - CPB
		 */
		if (isAjax(request))
			return;

		HttpSession session = SessionUtil.getHttpSession();
		Breadcrumbs breadcrumbs = (Breadcrumbs) session.getAttribute(SESSION_ATTRIBUTE_BREADCRUMBS);
		if (breadcrumbs == null) {
			String servletContextPath = request.getServletContext().getContextPath();

			breadcrumbs = new Breadcrumbs();
			breadcrumbs.navigate(servletContextPath + CommonController.URL_HOME, handler,
					CommonController.BREADCRUMB_HOME);
			session.setAttribute(SESSION_ATTRIBUTE_BREADCRUMBS, breadcrumbs);

		}

		String breadcrumb = (String) request.getAttribute(BreadcrumbsInterceptor.REQUEST_BREADCRUMB_KEY);

		if (breadcrumb != null) {
			breadcrumbs.navigate(request, handler, breadcrumb);
		}

		List<Breadcrumbs.Breadcrumb> currentBreadCrumbs = breadcrumbs.getBreadcrumbs();
		String lastButOneLinkHref = null;
		if ((currentBreadCrumbs != null) && (currentBreadCrumbs.size() > 1)) {
			// Get the last but one breadcrumb
			Breadcrumbs.Breadcrumb lastButOneDestination = currentBreadCrumbs.get(currentBreadCrumbs.size() - 2);
			if (lastButOneDestination != null) {
				lastButOneLinkHref = lastButOneDestination.getLink().getHref();
			}
		}

		if (modelAndView != null) {
			Map<String, Object> model = modelAndView.getModel();
			if (model != null) {
				model.put(MODEL_ATTRIBUTE_NAME, breadcrumbs);
				if (lastButOneLinkHref != null) {
					model.put(MOST_RECENT_BREADCRUMB_URL, lastButOneLinkHref);
				}
			}
		} else {
			log.debug("No modelAndView returned, the controller must have dealt with the response manually.");
		}
	}

}
