<%@ include file="../shared/inc_header.jsp"%>

<%--
	Expects the following params:
	
	- A "uniqueOrganizationSearchPopupId" param which should be a string of letters that
		uniquely identifies this organizationSearchPopup among other organizationSearchPopups on the same page
	- A "mode" param set to either "search", "add" or "duplicateCheck":
			If "search", search fields are shown and the callback is called as normal when the person clicks a result
			If "volunteerLink", only local orgs will be shown for facilities where the user also has the Create
				Volunteer permission
			If "donorLink"...
				
		If "mode" is unspecified, it defaults to "search".
	- A "includeInactiveOption" param, true/false, that specifies whether the checkbox for including inactive
		organizations is available to the user. If this is false or unspecified, the checkbox will be hidden
		and only active organizations will be shown.
	- A "callbackMethod" param which specifies the name of a JavaScript method defined by the
		-including- page which the organization popup will activate when the user selects a organization. The
		required method signature looks like:
		function someMethodWithUniqueName(organizationObj)
		
		The organizationObj parameter sent to the callback method is a javascript object containing the
		properties
		{
			id : <the ID of the organization selected>,
			[... other attributes of the Organization class marked for JsonProperty]
		}
		
	To activate this popup, call the method popupOrganizationSearch(uid) and pass in the same
	uniqueOrganizationSearchPopupId String above.
--%>

<c:set var="uid">
	<c:out value="${param.uniqueOrganizationSearchPopupId}" />
</c:set>

<script type="text/javascript">
	$(function() {
		initOrganizationSearchPopup({
			uid : "${uid}",
			maxResults : <fmt:message key="organizationSearch.maxResults" />,
			callbackMethod : <c:out value="${param.resultCallbackMethod}" default="null" />,
			includeInactiveOption : <c:out value="${param.includeInactiveOption}" default="false" />,
			mode : "<c:out value="${param.mode}" />"
		})
	})
</script>

<div id="organizationSearchDialog${uid}" style="display: none"
	title="Search for Organization">
	<table align="center">
		<tr valign="top" class="organizationSearchFields${uid}">
			<td>
				<table>
					<tr>
						<td align="right"><label for='organizationSearchName${uid}'>Name:</label></td>
						<td><input type="text" id="organizationSearchName${uid}"
							value="" title="Type an Organization name" /></td>
					</tr>
					<tr id="inactiveOptionRow${uid}" style="display: none">
						<td align="right"><input type="checkbox"
							id="organizationSearchIncludeInactive${uid}" value="true" /></td>
						<td>Include Inactive Organizations</td>
					</tr>
				</table>
			</td>
			<td>
				<table>
					<tr>
						<td align="right"><label
							for='organizationSearchAbbreviation${uid}'>Abbreviation:</label></td>
						<td><input type="text"
							id="organizationSearchAbbreviation${uid}" value=""
							title="Type an Organization abbreviation" /></td>
					</tr>
					<tr>
						<td align="right"><label for='organizationSearchCode${uid}'>Code:</label></td>
						<td><input type="number" id="organizationSearchCode${uid}"
							value="" title="Type an Organization code" /></td>
					</tr>
				</table>
			</td>
		</tr>
		<tr align="center">
			<td colspan="2"><a style="margin-left: 20px"
				class="buttonAnchor organizationSearchLink${uid} organizationSearchFields${uid}"
				tabIndex="0">Search</a></td>
		</tr>
	</table>
	<div id="organizationSearchNoResults${uid}" style="display: none">
		<table align="center" cellpadding="10">
			<tr>
				<td>Sorry, no organizations were found that matched the
					specified criteria.</td>
			</tr>
		</table>
	</div>
	<div id="organizationSearchMaxResults${uid}"
		style="display: none; padding: 10px" align="center">Your search
		exceeded the maximum results. Please enter additional search criteria.
	</div>
	<div id="organizationSearchResultsTable${uid}" style="display: none">
		<table id="organizationSearchResultsList${uid}" class="stripe"
			summary="List of Organizations">
			<thead>
				<tr id="organizationSearch${uid}FilterRow">
					<td width="30%" class="noborder"></td>
					<td width="50%" class="noborder"></td>
					<td width="7%" class="noborder"></td>
					<td width="7%" class="noborder"></td>
					<td width="6%" class="noborder"></td>
				</tr>
				<tr>
					<th width="30%" id="facilityHeaderCol${uid}" class="select-filter">Facility</th>
					<th width="50%">Name</th>
					<th width="7%" class="select-filter">Type</th>
					<th width="7%">Code</th>
					<th width="6%">Abbreviation</th>
				</tr>
			</thead>
		</table>
	</div>
</div>
