package gov.va.vss.service;

import javax.jms.Message;

import org.junit.After;
import org.junit.Before;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jms.core.JmsTemplate;
import org.springframework.jms.listener.adapter.MessageListenerAdapter;

import com.mockrunner.jms.DestinationManager;
import com.mockrunner.mock.jms.MockQueue;

import gov.va.vss.AbstractWebAppTest;

public abstract class AbstractJMSTest extends AbstractWebAppTest {

	@Autowired
	protected DestinationManager mockDestinationManager;

	protected MockQueue queue;

	@Before
	public void initAbstractJMSTest() {
		queue = mockDestinationManager.createQueue(getJmsTemplate().getDefaultDestinationName());
	}

	@After
	public void destroyAbstractJMSTest() {
		mockDestinationManager.removeQueue(getJmsTemplate().getDefaultDestinationName());
	}

	protected void go(Object listener) {
		while (!queue.isEmpty()) {
			Message message = getJmsTemplate().receive(getQueue());
			MessageListenerAdapter mla = new MessageListenerAdapter(listener);
			customizeMessageListenerAdapter(mla);
			mla.onMessage(message);
		}
	}

	/*
	 * Subclasses can override this to change default MessageConverter or
	 * response behavior - CPB
	 */
	protected void customizeMessageListenerAdapter(MessageListenerAdapter mla) {
	}

	protected abstract JmsTemplate getJmsTemplate();

	protected MockQueue getQueue() {
		return queue;
	}
}
